@@ -35,6 +35,7 @@ struct _MicrofeedTimeout {
 	unsigned long int milliseconds;
 	MicrofeedTimeoutCallback callback;
 	void* user_data;
+	int in_handler : 1;
 };
 
 struct _MicrofeedWatch {
@@ -72,6 +73,7 @@ static void toggle_watch(DBusWatch *timeout, void *data);
 static void handle_watch(MicrofeedMain* main, int fd, MicrofeedWatchType type, void* user_data);
 static void wakeup_poll(MicrofeedMain* microfeed_main);
 static void wakeup_main(void* data);
+static void real_remove_timeout(MicrofeedTimeout* timeout);
 
 /**
  * Instantiates a new main loop with a shared session-wide DBus connection.
@@ -192,13 +194,13 @@ void microfeed_main_loop(MicrofeedMain* microfeed_main) {
 	int milliseconds;
 	MicrofeedWatch* watch;
 	struct pollfd* pollfd;
-	MicrofeedTimeoutCallback timeout_callback;
-	void* user_data;
 	MicrofeedWatchType type;
 	MicrofeedWatchCallback watch_callback;
+	void* user_data;
 	int fd;
 	char buffer[1024];
 	ssize_t dummy;
+	MicrofeedTimeout* timeout;
 
 	microfeed_mutex_lock(microfeed_main->mutex);
 	
@@ -258,16 +260,16 @@ void microfeed_main_loop(MicrofeedMain* microfeed_main) {
 		gettimeofday(&timeval, NULL);
 		milliseconds = timeval.tv_sec * 1000 + timeval.tv_usec / 1000;
 		if (microfeed_main->timeouts && microfeed_main->timeouts->milliseconds < milliseconds) {
-			timeout_callback = microfeed_main->timeouts->callback;
-			user_data = microfeed_main->timeouts->user_data;
+			timeout = microfeed_main->timeouts;
+			timeout->in_handler = 1;
 
 			microfeed_mutex_unlock(microfeed_main->mutex);
 
-			timeout_callback(microfeed_main, user_data);
-			microfeed_main_remove_timeout(microfeed_main, microfeed_main->timeouts);
+			timeout->callback(microfeed_main, timeout->user_data);
 
 			microfeed_mutex_lock(microfeed_main->mutex);
-			
+
+			real_remove_timeout(timeout);		
 		}
 
 		if (microfeed_main->poll_fds->revents & POLLIN) {
@@ -423,25 +425,14 @@ MicrofeedWatch* microfeed_main_add_watch(MicrofeedMain* microfeed_main, int fd,
 /**
  * Removes a previously added timeout from the main loop.
  * 
- * TODO: There might be sitations when a timeout is already removed by the main loop itself, and thus
- * the application crashes when using this function.
- * 
  * @param microfeed_main Instantiated MicrofeedMain.
  * @param timeout A timeout previously added but not yet removed.
  */
 void microfeed_main_remove_timeout(MicrofeedMain* microfeed_main, MicrofeedTimeout* timeout) {
 	microfeed_mutex_lock(microfeed_main->mutex);
 
-	if (microfeed_main == timeout->main) {
-		if (timeout->previous) {
-			timeout->previous->next = timeout->next;
-		} else if (timeout->main) {
-			timeout->main->timeouts = timeout->next;
-		}
-		if (timeout->next) {
-			timeout->next->previous = timeout->previous;
-		}
-		microfeed_memory_free(timeout);	
+	if (microfeed_main == timeout->main && !timeout->in_handler) {
+		real_remove_timeout(timeout);
 	}
 
 	microfeed_mutex_unlock(microfeed_main->mutex);
@@ -617,3 +608,15 @@ static void wakeup_main(void* data) {
 	
 	microfeed_mutex_unlock(microfeed_main->mutex);
 }
+
+void real_remove_timeout(MicrofeedTimeout* timeout) {
+	if (timeout->previous) {
+		timeout->previous->next = timeout->next;
+	} else if (timeout->main) {
+		timeout->main->timeouts = timeout->next;
+	}
+	if (timeout->next) {
+		timeout->next->previous = timeout->previous;
+	}
+	microfeed_memory_free(timeout);	
+}
