@@ -39,11 +39,15 @@
  */
 typedef struct _MicrofeedWeakReference MicrofeedWeakReference;
 
+typedef void (*MicrofeedFreeCallback)(void* referenced, void* user_data);
+
 MicrofeedWeakReference* microfeed_weak_reference_new(void* referenced, MicrofeedWeakReference* existing_weak_reference);
 void microfeed_weak_reference_free(MicrofeedWeakReference* weak_refeference);
 void* microfeed_weak_reference_get_impl(MicrofeedWeakReference* weak_reference);
 #define microfeed_weak_reference_get(w, t) ((t*)microfeed_weak_reference_get_impl(w))
 void microfeed_weak_reference_invalidate(MicrofeedWeakReference* weak_reference);
+MicrofeedWeakReference* microfeed_weak_reference_add_free_callback(MicrofeedWeakReference* weak_reference, void* referenced, MicrofeedFreeCallback free_callback, void* user_data);
+void microfeed_weak_reference_remove_free_callback(MicrofeedWeakReference* weak_reference, MicrofeedFreeCallback free_callback, void* user_data);
 
 #define microfeed_memory_allocate(t) ((t*)microfeed_memory_allocate_bytes(sizeof(t)))
 void* microfeed_memory_allocate_bytes(size_t size);
