@@ -123,14 +123,14 @@ void microfeed_database_environment_unref(MicrofeedDatabaseEnvironment* database
 		microfeed_database_environment_free(database_environment);
 	} else {
 
-			microfeed_mutex_unlock(database_environment->mutex);
+		microfeed_mutex_unlock(database_environment->mutex);
 	}		
 }
 	
 MicrofeedDatabase* microfeed_database_environment_get_database(MicrofeedDatabaseEnvironment* database_environment, const char* name, MicrofeedDatabaseCompareFunction compare_function) {
 	MicrofeedDatabase* database;
 	
-	microfeed_mutex_lock(database_environment->mutex);
+	lock_environment_for_writing(database_environment);
 
 	if ((database = microfeed_store_get(database_environment->databases, name, MicrofeedDatabase))) {
 		microfeed_database_ref(database);
@@ -156,7 +156,7 @@ MicrofeedDatabase* microfeed_database_environment_get_database(MicrofeedDatabase
 		}
 	}
 
-	microfeed_mutex_unlock(database_environment->mutex);
+	unlock_environment(database_environment);
 	
 	return database;
 }
@@ -165,7 +165,7 @@ MicrofeedDatabaseIndex* microfeed_database_get_index(MicrofeedDatabase* database
 	MicrofeedDatabaseIndex* database_index;
 	char* full_name;
 	
-	microfeed_mutex_lock(database->mutex);
+	lock_environment_for_writing(database->database_environment);
 
 	for (database_index = database->indices; database_index; database_index = database_index->next) {
 		if (!strcmp(database_index->name, name)) {
@@ -195,7 +195,7 @@ MicrofeedDatabaseIndex* microfeed_database_get_index(MicrofeedDatabase* database
 		free(full_name);
 	}
 
-	microfeed_mutex_unlock(database->mutex);
+	unlock_environment(database->database_environment);
 	
 	return database_index;	
 }
