@@ -28,8 +28,6 @@
 #include <stdio.h>
 #include <stdint.h>
 
-typedef struct _SendItemDataData SendItemDataData;
-
 struct _MicrofeedFeed {
 	unsigned int reference_count;
 	MicrofeedWeakReference* weak_reference;
@@ -49,7 +47,7 @@ struct _MicrofeedFeed {
 	MicrofeedDatabase* item_data_database;
 	MicrofeedMutex* mutex;
 	int updating;
-	MicrofeedStore* send_item_data_datas;
+	MicrofeedStore* get_item_data_mutexes;
 };
 
 struct _MicrofeedFeedIterator {
@@ -63,34 +61,15 @@ struct _MicrofeedFeedIterator {
 };
 
 typedef struct {
-	MicrofeedFeed* feed;
-	const char* start_uid;
-	const char* end_uid;
-	unsigned int max_count;
-	const char* bus_name;
-} RepublishingData;
-
-typedef struct {
-	MicrofeedFeed* feed;
-	MicrofeedItem* existing_item;
-	MicrofeedItem* new_item;
-} ModifyItemData;
-
-struct _SendItemDataData {
-	MicrofeedFeed* feed;
+	unsigned int reference_count;
 	char* uid;	
-	MicrofeedStore* bus_names;
-};
+	MicrofeedMutex* mutex;
+} GetItemDataMutex;
 
 static void index_function(void* key, const size_t key_size, void* data, const size_t data_size, void** index_key, size_t* index_key_size);
 static int compare_function(const void* key1, const size_t key1_size, const void* key2, const size_t key2_size);
-static void* update_thread_periodical(void* data);
-static void* update_thread_user_initiated(void* data);
-static void* republish_thread(void* user_data);
-static void* send_item_data_thread(void* data);
-static void* modify_item_thread(void* user_data);
 static void remove_old_items(MicrofeedFeed* feed);
-static const char* send_item_data_data_get_uid(SendItemDataData* send_item_data_data);
+static const char* get_item_data_mutex_get_uid(GetItemDataMutex* get_item_data_mutex);
 
 MicrofeedFeed* microfeed_feed_new(MicrofeedPublisher* publisher, const char* uri, const char* name, MicrofeedFeedPermission feed_permission, MicrofeedFeedCallbacks* callbacks, void* user_data) {
 	MicrofeedFeed* feed;
@@ -128,9 +107,9 @@ MicrofeedFeed* microfeed_feed_new(MicrofeedPublisher* publisher, const char* uri
 		database_name = microfeed_util_string_concatenate(uri, ":microfeed.item_data", NULL);
 		feed->item_data_database = microfeed_database_environment_get_database(microfeed_publisher_get_database_environment(publisher), database_name, NULL);
 		free(database_name);
+		feed->get_item_data_mutexes = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)get_item_data_mutex_get_uid);
 	}
 	feed->mutex = microfeed_mutex_new();
-	feed->send_item_data_datas = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)send_item_data_data_get_uid);
 
 	return feed;	
 }
@@ -138,6 +117,8 @@ MicrofeedFeed* microfeed_feed_new(MicrofeedPublisher* publisher, const char* uri
 /* Does NOT remove subscribers */
 void microfeed_feed_free(MicrofeedFeed* feed) {
 	MicrofeedFeedIterator* iterator;
+
+	microfeed_weak_reference_invalidate(feed->weak_reference);
 	
 	if (feed->callbacks.destroy) {
 		feed->callbacks.destroy(feed, feed->user_data);
@@ -152,6 +133,7 @@ void microfeed_feed_free(MicrofeedFeed* feed) {
 	microfeed_database_unref(feed->statuses_database);
 	if (feed->item_data_database) {
 		microfeed_database_unref(feed->item_data_database);
+		microfeed_store_free(feed->get_item_data_mutexes);
 	}
 	microfeed_mutex_free(feed->mutex);
 
@@ -163,7 +145,6 @@ void microfeed_feed_free(MicrofeedFeed* feed) {
 	feed->callbacks.update = NULL;
 	feed->subscriber_count = 0;
 	feed->iterators = NULL;
-	microfeed_weak_reference_invalidate(feed->weak_reference);
 	feed->weak_reference = NULL;
 
 	free(feed);	
@@ -204,22 +185,27 @@ MicrofeedWeakReference* microfeed_feed_get_weak_reference(MicrofeedFeed* feed) {
 	return weak_reference;
 }
 
+void microfeed_feed_add_free_callback(MicrofeedFeed* feed, MicrofeedFreeCallback free_callback, void* user_data) {
+	microfeed_mutex_lock(feed->mutex);
+
+	feed->weak_reference = microfeed_weak_reference_add_free_callback(feed->weak_reference, feed, free_callback, user_data);
+
+	microfeed_mutex_unlock(feed->mutex);
+}
+
+void microfeed_feed_remove_free_callback(MicrofeedFeed* feed, MicrofeedFreeCallback free_callback, void* user_data) {
+	microfeed_mutex_lock(feed->mutex);
+
+	microfeed_weak_reference_remove_free_callback(feed->weak_reference, free_callback, user_data);
+
+	microfeed_mutex_unlock(feed->mutex);
+}
+
 MicrofeedPublisher* microfeed_feed_get_publisher(MicrofeedFeed* feed) {
 	
 	return feed->publisher;
 }
 
-void microfeed_feed_stop_update(MicrofeedFeed* feed) {
-	microfeed_mutex_lock(feed->mutex);
-	
-	if (feed->updating && microfeed_publisher_get_singleton_update_thread(feed->publisher)) {
-		microfeed_publisher_send_feed_signal(feed->publisher, NULL, MICROFEED_SIGNAL_NAME_FEED_UPDATE_ENDED, feed->uri);
-		feed->updating = 0;
-	}
-
-	microfeed_mutex_unlock(feed->mutex);
-}
-
 void microfeed_feed_replace_item(MicrofeedFeed* feed, MicrofeedItem* item) {
 	const char* uid;
 	size_t uid_size;
@@ -354,27 +340,6 @@ void microfeed_feed_remove_items(MicrofeedFeed* feed, const char* start_uid, con
 	microfeed_mutex_unlock(feed->mutex);	
 }
 
-void microfeed_feed_add_subscriber(MicrofeedFeed* feed, const char* bus_name) {
-	microfeed_mutex_lock(feed->mutex);
-
-	feed->subscriber_count++;
-	feed->reference_count++;
-
-	microfeed_mutex_unlock(feed->mutex);
-}
-
-int microfeed_feed_get_subscriber_count(MicrofeedFeed* feed) {
-	int subscriber_count;
-
-	microfeed_mutex_lock(feed->mutex);
-
-	subscriber_count = feed->subscriber_count;
-
-	microfeed_mutex_unlock(feed->mutex);
-
-	return subscriber_count;
-}
-
 MicrofeedItem* microfeed_feed_get_item(MicrofeedFeed* feed, const char* uid) {
 	void* key;
 	size_t key_size;
@@ -481,22 +446,10 @@ MicrofeedFeedIterator* microfeed_feed_iterate_timeline(MicrofeedFeed* feed, cons
 	return iterator;
 }
 
-void microfeed_feed_remove_subscriber(MicrofeedFeed* feed, const char* bus_name) {
-	microfeed_mutex_lock(feed->mutex);
-
-	feed->subscriber_count--;
-	feed->reference_count--;
-	if (feed->reference_count == 0) {
-		microfeed_feed_free(feed);
-	} else {
-
-		microfeed_mutex_unlock(feed->mutex);
-	}
-}
-
 void microfeed_feed_update(MicrofeedFeed* feed, const char* bus_name) {
 	MicrofeedError* error;
 
+	microfeed_feed_ref(feed);
 	microfeed_mutex_lock(feed->mutex);
 
 	if (feed->updating) {
@@ -504,50 +457,78 @@ void microfeed_feed_update(MicrofeedFeed* feed, const char* bus_name) {
 			microfeed_publisher_send_feed_signal(feed->publisher, bus_name, MICROFEED_SIGNAL_NAME_FEED_UPDATE_STARTED, feed->uri);
 		}
 	} else {
-		feed->updating = 1;
 		microfeed_publisher_send_feed_signal(feed->publisher, NULL, MICROFEED_SIGNAL_NAME_FEED_UPDATE_STARTED, feed->uri);
+		feed->updating = 1;
 	
 		if (feed->callbacks.update) {
-			if (microfeed_publisher_get_singleton_update_thread(feed->publisher)) {
-				microfeed_mutex_unlock(feed->mutex);
-				
-				error = feed->callbacks.update(feed, (bus_name ? 1 : 0), feed->user_data);
 
-				microfeed_mutex_lock(feed->mutex);
+			microfeed_mutex_unlock(feed->mutex);
 				
-				if (error) {
-					microfeed_publisher_send_error_signal(feed->publisher, NULL, microfeed_error_get_name(error), feed->uri, NULL, microfeed_error_get_message(error));
-					microfeed_error_free(error);
-				}
-			} else {
-				feed->reference_count++;
-				if (bus_name) {
-					microfeed_publisher_queue_thread(feed->publisher, update_thread_user_initiated, feed);
-				} else {
-					microfeed_publisher_queue_thread(feed->publisher, update_thread_periodical, feed);
-				}
+			error = feed->callbacks.update(feed, (bus_name ? 1 : 0), feed->user_data);
+
+			microfeed_mutex_lock(feed->mutex);
+				
+			if (error) {
+				microfeed_publisher_send_error_signal(feed->publisher, NULL, microfeed_error_get_name(error), feed->uri, NULL, microfeed_error_get_message(error));
+				microfeed_error_free(error);
 			}
 		}
+
+		microfeed_publisher_send_feed_signal(feed->publisher, NULL, MICROFEED_SIGNAL_NAME_FEED_UPDATE_ENDED, feed->uri);
+		feed->updating = 0;
 	}
-	
+
 	microfeed_mutex_unlock(feed->mutex);
+	microfeed_feed_unref(feed);
 }
 
 void microfeed_feed_republish(MicrofeedFeed* feed, const char* start_uid, const char* end_uid, unsigned int max_count, const char* bus_name) {
-	RepublishingData* data;
+	unsigned int count;
+	MicrofeedDatabaseIterator* iterator;
+	const void* key;
+	size_t key_size;
+	const void* item_data;
+	size_t item_data_size;
+	uint64_t timestamp;
+	MicrofeedItem* item;
+	void* status_data;
+	size_t status_data_size;
+	char status;
 	
-	data = microfeed_memory_allocate(RepublishingData);
-	data->feed = feed;
-	data->start_uid = start_uid;
-	data->end_uid = end_uid;
-	data->max_count = max_count;
-	data->bus_name = bus_name;
+	microfeed_mutex_lock(feed->mutex);
 	
-	microfeed_publisher_queue_thread(feed->publisher, republish_thread, data);
+	microfeed_publisher_send_feed_signal(feed->publisher, bus_name, MICROFEED_SIGNAL_NAME_FEED_REPUBLISHING_STARTED, feed->uri);
+
+	count = 0;
+	for (iterator = microfeed_database_iterate(feed->database, end_uid, (end_uid ? strlen(end_uid) + 1 : 0), 1);
+	     microfeed_database_iterator_get(iterator, &key, &key_size, &item_data, &item_data_size);
+	     microfeed_database_iterator_next(iterator)) {
+	    timestamp = *((uint64_t*)key);
+		if (microfeed_database_get_data(feed->statuses_database, key + sizeof(uint64_t), key_size - sizeof(uint64_t), &status_data, &status_data_size)) {
+			status = *((char*)status_data);
+		} else {
+			status = (char)MICROFEED_ITEM_STATUS_NONE;
+		}
+		item = microfeed_item_new_with_status(key + sizeof(uint64_t), (time_t)timestamp, (MicrofeedItemStatus)status);
+		microfeed_item_demarshal_properties(item, item_data, item_data_size);
+		microfeed_publisher_send_item_signal(feed->publisher, bus_name, MICROFEED_SIGNAL_NAME_ITEM_REPUBLISHED, feed->uri, item);
+		microfeed_item_free(item);
+	    if (start_uid && strcmp(start_uid, key + sizeof(uint64_t)) <= 0) {
+	    	break;
+	    }
+	    if (++count == max_count) {
+	    	break;
+	    }
+	}
+	microfeed_database_iterator_free(iterator);
+
+	microfeed_publisher_send_feed_signal(feed->publisher, bus_name, MICROFEED_SIGNAL_NAME_FEED_REPUBLISHING_ENDED, feed->uri);
+
+	microfeed_mutex_unlock(feed->mutex);	
 }
 
-int microfeed_feed_set_item_status(MicrofeedFeed* feed, const char* uid, MicrofeedItemStatus status_to_set) {
-	int retval = 0;
+MicrofeedError* microfeed_feed_set_item_status(MicrofeedFeed* feed, const char* uid, MicrofeedItemStatus status_to_set) {
+	MicrofeedError* error = NULL;
 	size_t uid_size;
 	void* key;
 	size_t key_size;
@@ -559,36 +540,41 @@ int microfeed_feed_set_item_status(MicrofeedFeed* feed, const char* uid, Microfe
 
 	uid_size = strlen(uid) + 1;
 	if (microfeed_database_get_data(feed->statuses_database, uid, uid_size, &data, &data_size)) {
-		retval = 1;
 		status = *((char*)data);
 		if (!(status & (char)status_to_set)) {
 			if (status_to_set == MICROFEED_ITEM_STATUS_MARKED && feed->callbacks.mark_item) {
-				feed->callbacks.mark_item(feed, uid, 1, feed->user_data);
+				error = feed->callbacks.mark_item(feed, uid, 1, feed->user_data);
 			}
-			status |= (char)(status_to_set);
-			microfeed_publisher_send_status_changed_signal(feed->publisher, NULL, feed->uri, uid, status);
-			if (status) {
-				microfeed_database_replace_data(feed->statuses_database, uid, uid_size, &status, sizeof(char));
-			} else {
-				microfeed_database_remove_data(feed->statuses_database, uid, uid_size);
+			if (!error) {
+				status |= (char)(status_to_set);
+				microfeed_publisher_send_status_changed_signal(feed->publisher, NULL, feed->uri, uid, status);
+				if (status) {
+					microfeed_database_replace_data(feed->statuses_database, uid, uid_size, &status, sizeof(char));
+				} else {
+					microfeed_database_remove_data(feed->statuses_database, uid, uid_size);
+				}
 			}
 		}
 	} else if (microfeed_database_index_get_data(feed->database_index, uid, uid_size, &key, &key_size, &data, &data_size)) {
 		if (status_to_set == MICROFEED_ITEM_STATUS_MARKED && feed->callbacks.mark_item) {
-			feed->callbacks.mark_item(feed, uid, 1, feed->user_data);
+			error = feed->callbacks.mark_item(feed, uid, 1, feed->user_data);
 		}
-		status = (char)status_to_set;
-		microfeed_publisher_send_status_changed_signal(feed->publisher, NULL, feed->uri, uid, status);
-		microfeed_database_replace_data(feed->statuses_database, uid, uid_size, &status, sizeof(char));
+		if (!error) {
+			status = (char)status_to_set;
+			microfeed_publisher_send_status_changed_signal(feed->publisher, NULL, feed->uri, uid, status);
+			microfeed_database_replace_data(feed->statuses_database, uid, uid_size, &status, sizeof(char));
+		}
+	} else {
+		error = microfeed_error_new(MICROFEED_ERROR_NO_SUCH_ITEM, "No such item when setting status.");
 	}
 
 	microfeed_mutex_unlock(feed->mutex);	
 
-	return retval;
+	return error;
 }
 
-int microfeed_feed_unset_item_status(MicrofeedFeed* feed, const char* uid, MicrofeedItemStatus status_to_unset) {
-	int retval = 0;
+MicrofeedError* microfeed_feed_unset_item_status(MicrofeedFeed* feed, const char* uid, MicrofeedItemStatus status_to_unset) {
+	MicrofeedError* error = NULL;
 	size_t uid_size;
 	void* data;
 	size_t data_size;
@@ -598,28 +584,32 @@ int microfeed_feed_unset_item_status(MicrofeedFeed* feed, const char* uid, Micro
 
 	uid_size = strlen(uid) + 1;
 	if (microfeed_database_get_data(feed->statuses_database, uid, uid_size, &data, &data_size)) {
-		retval = 1;
 		status = *((char*)data);
 		if (status & (char)status_to_unset) {
 			if (status_to_unset == MICROFEED_ITEM_STATUS_MARKED && feed->callbacks.mark_item) {
-				feed->callbacks.mark_item(feed, uid, 0, feed->user_data);
+				error = feed->callbacks.mark_item(feed, uid, 0, feed->user_data);
 			}
-			status &= (char)(~status_to_unset);
-			microfeed_publisher_send_status_changed_signal(feed->publisher, NULL, feed->uri, uid, status);
-			if (status) {
-				microfeed_database_replace_data(feed->statuses_database, uid, uid_size, &status, sizeof(char));
-			} else {
-				microfeed_database_remove_data(feed->statuses_database, uid, uid_size);
+			if (!error) {
+				status &= (char)(~status_to_unset);
+				microfeed_publisher_send_status_changed_signal(feed->publisher, NULL, feed->uri, uid, status);
+				if (status) {
+					microfeed_database_replace_data(feed->statuses_database, uid, uid_size, &status, sizeof(char));
+				} else {
+					microfeed_database_remove_data(feed->statuses_database, uid, uid_size);
+				}
 			}
 		}			
+	} else {
+		error = microfeed_error_new(MICROFEED_ERROR_NO_SUCH_ITEM, "No such item when unsetting status.");
 	}
 
 	microfeed_mutex_unlock(feed->mutex);	
 
-	return retval;
+	return error;
 }
 
-void microfeed_feed_unset_item_statuses(MicrofeedFeed* feed, const char* start_uid, const char* end_uid, MicrofeedItemStatus status_to_unset) {
+MicrofeedError* microfeed_feed_unset_item_statuses(MicrofeedFeed* feed, const char* start_uid, const char* end_uid, MicrofeedItemStatus status_to_unset) {
+	MicrofeedError* error = NULL;
 	size_t uid_size;
 	MicrofeedDatabaseIterator* iterator;
 	const void* key;
@@ -632,62 +622,103 @@ void microfeed_feed_unset_item_statuses(MicrofeedFeed* feed, const char* start_u
 	
 	uid_size = (start_uid ? strlen(start_uid) + 1 : 0);
 	for (iterator = microfeed_database_iterate(feed->statuses_database, start_uid, uid_size, 0);
-	     (microfeed_database_iterator_get(iterator, &key, &key_size, &data, &data_size)) && (!end_uid || strcmp ((const char*)key, end_uid) <= 0);
+	     !error && (microfeed_database_iterator_get(iterator, &key, &key_size, &data, &data_size)) && (!end_uid || strcmp ((const char*)key, end_uid) <= 0);
 	     microfeed_database_iterator_next(iterator)) {
 		status = *((char*)data);
 		if (status & (char)status_to_unset) {
 			if (status_to_unset == MICROFEED_ITEM_STATUS_MARKED && feed->callbacks.mark_item) {
-				feed->callbacks.mark_item(feed, key, 0, feed->user_data);
+				error = feed->callbacks.mark_item(feed, key, 0, feed->user_data);
 			}
-			status &= (char)(~status_to_unset);
-			microfeed_publisher_send_status_changed_signal(feed->publisher, NULL, feed->uri, (const char*)key, status);
-			if (status) {
-				microfeed_database_replace_data(feed->statuses_database, key, key_size, &status, sizeof(char));
-			} else {
-				microfeed_database_remove_data(feed->statuses_database, key, key_size);
+			if (!error) {
+				status &= (char)(~status_to_unset);
+				microfeed_publisher_send_status_changed_signal(feed->publisher, NULL, feed->uri, (const char*)key, status);
+				if (status) {
+					microfeed_database_replace_data(feed->statuses_database, key, key_size, &status, sizeof(char));
+				} else {
+					microfeed_database_remove_data(feed->statuses_database, key, key_size);
+				}
 			}
 		}     	
 	}
 	microfeed_database_iterator_free(iterator);
 	
 	microfeed_mutex_unlock(feed->mutex);	
+
+	return error;
 }
 
-void microfeed_feed_send_item_data(MicrofeedFeed* feed, const char* uid, const char* bus_name) {
+MicrofeedError* microfeed_feed_send_item_data(MicrofeedFeed* feed, const char* uid, const char* bus_name) {
+	MicrofeedError* error = NULL;
 	void* key;
 	size_t key_size;
 	void* data;
 	size_t data_size;
-	SendItemDataData* sid_data;
-	const char* existing_bus_name;
+	GetItemDataMutex* gidm;
 	
+	microfeed_feed_ref(feed);
 	microfeed_mutex_lock(feed->mutex);
 	
 	if (!feed->item_data_database) {
-		microfeed_publisher_send_error_signal(feed->publisher, bus_name, MICROFEED_ERROR_NO_ITEM_DATA, feed->uri, uid, "Feed does not support item data");
+		error = microfeed_error_new(MICROFEED_ERROR_NO_ITEM_DATA, "Feed does not support item data");
+	} else if (microfeed_database_get_data(feed->item_data_database, uid, strlen(uid) + 1, &data, &data_size) && data_size > sizeof(uint64_t)) {
+		microfeed_publisher_send_item_data_signal(feed->publisher, bus_name, feed->uri, uid, data + sizeof(uint64_t), data_size - sizeof(uint64_t));
+		free(data);
 	} else if (!microfeed_database_index_get_data(feed->database_index, uid, strlen(uid) + 1, &key, &key_size, &data, &data_size)) {
-		microfeed_publisher_send_error_signal(feed->publisher, bus_name, MICROFEED_ERROR_NO_SUCH_ITEM, feed->uri, uid, "Item does not exist");		
-	}  else {
-		if (microfeed_database_get_data(feed->item_data_database, uid, strlen(uid) + 1, &data, &data_size) && data_size > sizeof(uint64_t)) {
-			microfeed_publisher_send_item_data_signal(feed->publisher, bus_name, feed->uri, uid, data + sizeof(uint64_t), data_size - sizeof(uint64_t));
-			free(data);
-		} else {
-			if (!(sid_data = microfeed_store_get(feed->send_item_data_datas, uid, SendItemDataData))) {
-				sid_data = microfeed_memory_allocate(SendItemDataData);
-				sid_data->uid = strdup(uid);
-				sid_data->feed = feed;
-				sid_data->bus_names = microfeed_store_new_sorted(microfeed_store_compare_keys_direct, microfeed_store_get_key_direct);
-				microfeed_store_insert(feed->send_item_data_datas, sid_data);
+		error = microfeed_error_new(MICROFEED_ERROR_NO_SUCH_ITEM, "Item does not exist");
+	} else {
+		free(key);
+		free(data);
+		
+		if ((gidm = microfeed_store_get(feed->get_item_data_mutexes, uid, GetItemDataMutex))) {
+			gidm->reference_count++;
 
-				microfeed_publisher_queue_thread(feed->publisher, send_item_data_thread, sid_data);
+			microfeed_mutex_unlock(feed->mutex);
+
+			microfeed_mutex_lock(gidm->mutex);
+
+			microfeed_mutex_lock(feed->mutex);
+			if (microfeed_database_get_data(feed->item_data_database, uid, strlen(uid) + 1, &data, &data_size) && data_size > sizeof(uint64_t)) {
+				microfeed_publisher_send_item_data_signal(feed->publisher, bus_name, feed->uri, uid, data + sizeof(uint64_t), data_size - sizeof(uint64_t));
+				free(data);
+			} else {
+				error = microfeed_error_new(MICROFEED_ERROR_NO_ITEM_DATA, "Item data does not exist");
 			}
-			if (!(existing_bus_name = microfeed_store_get(sid_data->bus_names, bus_name, const char))) {
-				microfeed_store_insert(sid_data->bus_names, strdup(bus_name));
+		} else {
+			gidm = microfeed_memory_allocate(GetItemDataMutex);
+			gidm->reference_count = 1;
+			gidm->uid = strdup(uid);
+			gidm->mutex = microfeed_mutex_new();
+			microfeed_mutex_lock(gidm->mutex);
+			microfeed_store_insert(feed->get_item_data_mutexes, gidm);
+
+			microfeed_mutex_unlock(feed->mutex);
+
+			error = feed->callbacks.download_item_data(feed, uid, &data, &data_size, feed->user_data);
+
+			microfeed_mutex_lock(feed->mutex);
+
+			if (!error) {
+				microfeed_publisher_send_item_data_signal(feed->publisher, bus_name, feed->uri, uid, data, data_size);
+
+				data = realloc(data, data_size + sizeof(uint64_t));
+				memmove(data + sizeof(uint64_t), data, data_size);
+				*((uint64_t*)data) = 1;
+				microfeed_database_replace_data(feed->item_data_database, uid, strlen(uid) + 1, data, data_size + sizeof(uint64_t));
+				free(data);
 			}
 		}
+		gidm->reference_count--;
+		if (gidm->reference_count == 0) {
+			microfeed_store_remove(feed->get_item_data_mutexes, gidm);
+			microfeed_mutex_free(gidm->mutex);
+			microfeed_memory_free(gidm);
+		} else {
+			microfeed_mutex_unlock(gidm->mutex);
+		}
 	}
 	
 	microfeed_mutex_unlock(feed->mutex);	
+	microfeed_feed_unref(feed);
 }
 
 void microfeed_feed_ref_item_data(MicrofeedFeed* feed, const char* uid) {
@@ -893,30 +924,32 @@ int microfeed_feed_iterator_jump_and_remove_previous_items(MicrofeedFeedIterator
 	return retval;
 }
 
-void microfeed_feed_call_modify_item_callback(MicrofeedFeed* feed, MicrofeedItem* existing_item, MicrofeedItem* new_item) {
-	ModifyItemData* data;
+MicrofeedError* microfeed_feed_call_modify_item_callback(MicrofeedFeed* feed, MicrofeedItem* existing_item, MicrofeedItem* new_item) {
+	MicrofeedError* error;
 	
-	data = microfeed_memory_allocate(ModifyItemData);
-	data->feed = feed;
-	if (existing_item) {
-		data->existing_item = microfeed_item_duplicate(existing_item);
-	}
-	if (new_item) {
-		data->new_item = microfeed_item_duplicate(new_item);
-	}
-
+	microfeed_feed_ref(feed);
 	microfeed_mutex_lock(feed->mutex);
 
 /* TODO: There must NOT be on-going update. Wait somewhere until update ends... */
 
 	if (feed->callbacks.modify_item) {
-		feed->reference_count++;
-		microfeed_publisher_queue_thread(feed->publisher, modify_item_thread, data);
+
+		microfeed_mutex_unlock(feed->mutex);
+
+		error = feed->callbacks.modify_item(feed, existing_item, new_item, feed->user_data);
+
+		microfeed_mutex_lock(feed->mutex);
+	
+		if (error) {
+			microfeed_publisher_send_error_signal(feed->publisher, NULL, microfeed_error_get_name(error), feed->uri, microfeed_item_get_uid((existing_item ? existing_item : new_item)), microfeed_error_get_message(error));
+			microfeed_error_free(error);
+		}
 	} else {
 		/* TODO: Send CannotModifyItem signal */
 	}
 	
 	microfeed_mutex_unlock(feed->mutex);
+	microfeed_feed_unref(feed);
 }
 
 static void index_function(void* key, const size_t key_size, void* data, const size_t data_size, void** index_key, size_t* index_key_size) {
@@ -936,184 +969,6 @@ static int compare_function(const void* key1, const size_t key1_size, const void
 	return retvalue;
 }
 
-static void* update_thread_periodical(void* data) {
-	MicrofeedFeed* feed;
-	MicrofeedError* error;
-	
-	feed = (MicrofeedFeed*)data;
-	error = feed->callbacks.update(feed, 0, feed->user_data);
-
-	microfeed_mutex_lock(feed->mutex);
-
-	if (error) {
-		microfeed_publisher_send_error_signal(feed->publisher, NULL, microfeed_error_get_name(error), feed->uri, NULL, microfeed_error_get_message(error));
-		microfeed_error_free(error);
-	} else {
-		remove_old_items(feed);
-	}
-	microfeed_publisher_send_feed_signal(feed->publisher, NULL, MICROFEED_SIGNAL_NAME_FEED_UPDATE_ENDED, feed->uri);
-	feed->updating = 0;
-	feed->reference_count--;
-	if (feed->reference_count == 0) {
-		microfeed_feed_free(feed);
-	} else {
-
-		microfeed_mutex_unlock(feed->mutex);
-	}
-
-	return NULL;
-}
-
-static void* update_thread_user_initiated(void* data) {
-	MicrofeedFeed* feed;
-	MicrofeedError* error;
-	
-	feed = (MicrofeedFeed*)data;
-	error = feed->callbacks.update(feed, 1, feed->user_data);
-
-	microfeed_mutex_lock(feed->mutex);
-	
-	if (error) {
-		microfeed_publisher_send_error_signal(feed->publisher, NULL, microfeed_error_get_name(error), feed->uri, NULL, microfeed_error_get_message(error));
-		microfeed_error_free(error);
-	} else {
-		remove_old_items(feed);	
-	}
-	microfeed_publisher_send_feed_signal(feed->publisher, NULL, MICROFEED_SIGNAL_NAME_FEED_UPDATE_ENDED, feed->uri);
-	feed->updating = 0;
-	feed->reference_count--;
-	if (feed->reference_count == 0) {
-		microfeed_feed_free(feed);
-	} else {
-
-		microfeed_mutex_unlock(feed->mutex);
-	}
-	
-	return NULL;
-}
-
-static void* republish_thread(void* user_data) {
-	RepublishingData* data;
-	unsigned int count;
-	MicrofeedDatabaseIterator* iterator;
-	const void* key;
-	size_t key_size;
-	const void* item_data;
-	size_t item_data_size;
-	uint64_t timestamp;
-	MicrofeedItem* item;
-	void* status_data;
-	size_t status_data_size;
-	char status;
-	
-
-	data = (RepublishingData*)user_data;
-
-	microfeed_mutex_lock(data->feed->mutex);
-	
-	microfeed_publisher_send_feed_signal(data->feed->publisher, data->bus_name, MICROFEED_SIGNAL_NAME_FEED_REPUBLISHING_STARTED, data->feed->uri);
-
-	count = 0;
-	for (iterator = microfeed_database_iterate(data->feed->database, data->end_uid, (data->end_uid ? strlen(data->end_uid) + 1 : 0), 1);
-	     microfeed_database_iterator_get(iterator, &key, &key_size, &item_data, &item_data_size);
-	     microfeed_database_iterator_next(iterator)) {
-	    timestamp = *((uint64_t*)key);
-		if (microfeed_database_get_data(data->feed->statuses_database, key + sizeof(uint64_t), key_size - sizeof(uint64_t), &status_data, &status_data_size)) {
-			status = *((char*)status_data);
-		} else {
-			status = (char)MICROFEED_ITEM_STATUS_NONE;
-		}
-		item = microfeed_item_new_with_status(key + sizeof(uint64_t), (time_t)timestamp, (MicrofeedItemStatus)status);
-		microfeed_item_demarshal_properties(item, item_data, item_data_size);
-		microfeed_publisher_send_item_signal(data->feed->publisher, data->bus_name, MICROFEED_SIGNAL_NAME_ITEM_REPUBLISHED, data->feed->uri, item);
-		microfeed_item_free(item);
-	    if (data->start_uid && strcmp(data->start_uid, key + sizeof(uint64_t)) <= 0) {
-	    	break;
-	    }
-	    if (++count == data->max_count) {
-	    	break;
-	    }
-	}
-	microfeed_database_iterator_free(iterator);
-
-	microfeed_publisher_send_feed_signal(data->feed->publisher, data->bus_name, MICROFEED_SIGNAL_NAME_FEED_REPUBLISHING_ENDED, data->feed->uri);
-
-	microfeed_mutex_unlock(data->feed->mutex);	
-	
-	return NULL;
-}
-
-static void* send_item_data_thread(void* data) {
-	SendItemDataData* sid_data;
-	MicrofeedError* error;
-	MicrofeedStoreIterator* iterator;
-	void* item_data;
-	size_t item_data_size;
-	char* bus_name;
-
-	sid_data = (SendItemDataData*)data;
-	error = sid_data->feed->callbacks.download_item_data(sid_data->feed, sid_data->uid, &item_data, &item_data_size, sid_data->feed->user_data);
-
-	microfeed_mutex_lock(sid_data->feed->mutex);
-	
-	if (error) {
-		for (iterator = microfeed_store_iterate(sid_data->bus_names, NULL);
-		     (bus_name = microfeed_store_iterator_get(iterator, char));
-		     microfeed_store_iterator_next(iterator)) {		     	
-			microfeed_publisher_send_error_signal(sid_data->feed->publisher, bus_name, microfeed_error_get_name(error), sid_data->feed->uri, NULL, microfeed_error_get_message(error));
-			free(bus_name);
-		}
-		microfeed_store_iterator_free(iterator);
-		microfeed_error_free(error);
-	} else {
-		for (iterator = microfeed_store_iterate(sid_data->bus_names, NULL);
-		     (bus_name = microfeed_store_iterator_get(iterator, char));
-		     microfeed_store_iterator_next(iterator)) {		     	
-			microfeed_publisher_send_item_data_signal(sid_data->feed->publisher, bus_name, sid_data->feed->uri, sid_data->uid, item_data, item_data_size);
-			free(bus_name);
-		}
-		microfeed_store_iterator_free(iterator);
-
-		item_data = realloc(item_data, item_data_size + sizeof(uint64_t));
-		memmove(item_data + sizeof(uint64_t), item_data, item_data_size);
-		item_data_size += sizeof(uint64_t);
-		*((uint64_t*)item_data) = 1;
-		microfeed_database_replace_data(sid_data->feed->item_data_database, sid_data->uid, strlen(sid_data->uid) + 1, item_data, item_data_size);
-		free(item_data);
-	}
-	microfeed_store_remove(sid_data->feed->send_item_data_datas, sid_data);
-	
-	microfeed_mutex_unlock(sid_data->feed->mutex);
-
-	microfeed_store_free(sid_data->bus_names);
-	free(sid_data->uid);
-	microfeed_memory_free(sid_data);
-	
-	return NULL;
-}
-
-static void* modify_item_thread(void* user_data) {
-	ModifyItemData* data;
-	MicrofeedError* error;
-	
-	data = (ModifyItemData*)user_data;
-	error = data->feed->callbacks.modify_item(data->feed, data->existing_item, data->new_item, data->feed->user_data);
-
-	microfeed_mutex_lock(data->feed->mutex);
-	
-	if (error) {
-		microfeed_publisher_send_error_signal(data->feed->publisher, NULL, microfeed_error_get_name(error), data->feed->uri, microfeed_item_get_uid((data->existing_item ? data->existing_item : data->new_item)), microfeed_error_get_message(error));
-		microfeed_error_free(error);
-	}
-	
-	microfeed_item_free(data->existing_item);
-	microfeed_item_free(data->new_item);
-
-	microfeed_mutex_unlock(data->feed->mutex);
-
-	return NULL;	
-}
-
 static void remove_old_items(MicrofeedFeed* feed) {
 	time_t timestamp;
 	int keep_old_items;
@@ -1125,7 +980,7 @@ static void remove_old_items(MicrofeedFeed* feed) {
 	
 }
 
-static const char* send_item_data_data_get_uid(SendItemDataData* send_item_data_data) {
+static const char* get_item_data_mutex_get_uid(GetItemDataMutex* get_item_data_mutex) {
 	
-	return send_item_data_data->uid;
+	return get_item_data_mutex->uid;
 }
