@@ -120,29 +120,26 @@ void microfeed_feed_free(MicrofeedFeed* feed);
 MicrofeedFeed* microfeed_feed_ref(MicrofeedFeed* feed);
 void microfeed_feed_unref(MicrofeedFeed* feed);
 MicrofeedWeakReference* microfeed_feed_get_weak_reference(MicrofeedFeed* feed);
+void microfeed_feed_add_free_callback(MicrofeedFeed* feed, MicrofeedFreeCallback free_callback, void* user_data);
+void microfeed_feed_remove_free_callback(MicrofeedFeed* feed, MicrofeedFreeCallback free_callback, void* user_data);
 
 MicrofeedPublisher* microfeed_feed_get_publisher(MicrofeedFeed* feed);
-int microfeed_feed_is_using_threads(MicrofeedFeed* feed);
-void microfeed_feed_stop_update(MicrofeedFeed* feed);
-void microfeed_feed_replace_item(MicrofeedFeed* feed, MicrofeedItem* item);
-void microfeed_feed_remove_item(MicrofeedFeed* feed, const char* uid);
-void microfeed_feed_remove_items(MicrofeedFeed* feed, const char* start_uid, const char* end_uid);
-void microfeed_feed_add_subscriber(MicrofeedFeed* feed, const char* bus_name);
-MicrofeedItem* microfeed_feed_get_item(MicrofeedFeed* feed, const char* uid);
-int microfeed_feed_get_subscriber_count(MicrofeedFeed* feed);
 const char* microfeed_feed_get_uri(MicrofeedFeed* feed);
 const char* microfeed_feed_get_name(MicrofeedFeed* feed);
 void microfeed_feed_set_name(MicrofeedFeed* feed, const char* name);
 MicrofeedItem* microfeed_feed_get_metadata_item(MicrofeedFeed* feed);
+void microfeed_feed_replace_item(MicrofeedFeed* feed, MicrofeedItem* item);
+void microfeed_feed_remove_item(MicrofeedFeed* feed, const char* uid);
+void microfeed_feed_remove_items(MicrofeedFeed* feed, const char* start_uid, const char* end_uid);
+MicrofeedItem* microfeed_feed_get_item(MicrofeedFeed* feed, const char* uid);
+MicrofeedError* microfeed_feed_send_item_data(MicrofeedFeed* feed, const char* uid, const char* bus_name);
 MicrofeedFeedIterator* microfeed_feed_iterate(MicrofeedFeed* feed, const char* start_uid, int backwards);
 MicrofeedFeedIterator* microfeed_feed_iterate_timeline(MicrofeedFeed* feed, time_t start_timestamp, int backwards);
-void microfeed_feed_remove_subscriber(MicrofeedFeed* feed, const char* bus_name);
 void microfeed_feed_update(MicrofeedFeed* feed, const char* bus_name);
 void microfeed_feed_republish(MicrofeedFeed* feed, const char* start_uid, const char* end_uid, unsigned int max_count, const char* bus_name);
-int microfeed_feed_set_item_status(MicrofeedFeed* feed, const char* uid, MicrofeedItemStatus status_to_set);
-int microfeed_feed_unset_item_status(MicrofeedFeed* feed, const char* uid, MicrofeedItemStatus status_to_unset);
-void microfeed_feed_unset_item_statuses(MicrofeedFeed* feed, const char* start_uid, const char* end_uid, MicrofeedItemStatus status_to_unset);
-void microfeed_feed_send_item_data(MicrofeedFeed* feed, const char* uid, const char* bus_name);
+MicrofeedError* microfeed_feed_set_item_status(MicrofeedFeed* feed, const char* uid, MicrofeedItemStatus status_to_set);
+MicrofeedError* microfeed_feed_unset_item_status(MicrofeedFeed* feed, const char* uid, MicrofeedItemStatus status_to_unset);
+MicrofeedError* microfeed_feed_unset_item_statuses(MicrofeedFeed* feed, const char* start_uid, const char* end_uid, MicrofeedItemStatus status_to_unset);
 void microfeed_feed_ref_item_data(MicrofeedFeed* feed, const char* uid);
 void microfeed_feed_unref_item_data(MicrofeedFeed* feed, const char* uid);
 MicrofeedFeedPermission microfeed_feed_permission_from_string(const char* string);
@@ -160,6 +157,6 @@ int microfeed_feed_iterator_jump_and_remove_previous_items(MicrofeedFeedIterator
  */
 
 /* For internal use only. */
-void microfeed_feed_call_modify_item_callback(MicrofeedFeed* feed, MicrofeedItem* existing_item, MicrofeedItem* new_item);
+MicrofeedError* microfeed_feed_call_modify_item_callback(MicrofeedFeed* feed, MicrofeedItem* existing_item, MicrofeedItem* new_item);
 
 #endif
