@@ -329,6 +329,95 @@ MicrofeedJson* microfeed_json_get_array_by_path(MicrofeedJson* json, const char*
 	return array;
 }
 
+long long int microfeed_json_get_integer(MicrofeedJson* json, const char* name) {
+	long long int integer = 0;
+	Member* member;
+	
+	if (name) {
+		if ((member = microfeed_store_get(json->members, name, Member)) && member->type == MICROFEED_JSON_TYPE_INTEGER) {
+			integer = member->integer;
+		}
+	}
+	
+	return integer;
+}
+
+long long int microfeed_json_get_integer_by_index(MicrofeedJson* json, unsigned int index) {
+	long long int integer = 0;;
+	Member* member;
+	
+	if ((member = microfeed_store_get_index(json->members, index, Member)) && member->type == MICROFEED_JSON_TYPE_INTEGER) {
+		integer = member->integer;
+	}
+	
+	return integer;
+}
+
+long long int microfeed_json_get_integer_by_path(MicrofeedJson* json, const char* name, ...) {
+	long long int integer = 0;
+	Member* member;
+	va_list argp;
+	const char* s;
+	
+	if ((member = microfeed_store_get(json->members, name, Member))) {
+		va_start(argp, name);
+		while ((s = va_arg(argp, const char*)) && member->type == MICROFEED_JSON_TYPE_OBJECT &&
+		       (member = microfeed_store_get(member->object->members, s, Member))) {
+		}
+		va_end(argp);
+		if (!s && member->type == MICROFEED_JSON_TYPE_INTEGER) {
+			integer = member->integer;
+		}
+	}
+		
+	return integer;
+}
+
+double microfeed_json_get_decimal(MicrofeedJson* json, const char* name) {
+	double decimal = 0;
+	Member* member;
+	
+	if (name) {
+		if ((member = microfeed_store_get(json->members, name, Member)) && member->type == MICROFEED_JSON_TYPE_DECIMAL) {
+			decimal = member->decimal;
+		}
+	}
+	
+	return decimal;
+}
+
+double microfeed_json_get_decimal_by_index(MicrofeedJson* json, unsigned int index) {
+	double decimal = 0;;
+	Member* member;
+	
+	if ((member = microfeed_store_get_index(json->members, index, Member)) && member->type == MICROFEED_JSON_TYPE_DECIMAL) {
+		decimal = member->decimal;
+	}
+	
+	return decimal;
+
+}
+
+double microfeed_json_get_decimal_by_path(MicrofeedJson* json, const char* name, ...) {
+	double decimal = 0;
+	Member* member;
+	va_list argp;
+	const char* s;
+	
+	if ((member = microfeed_store_get(json->members, name, Member))) {
+		va_start(argp, name);
+		while ((s = va_arg(argp, const char*)) && member->type == MICROFEED_JSON_TYPE_OBJECT &&
+		       (member = microfeed_store_get(member->object->members, s, Member))) {
+		}
+		va_end(argp);
+		if (!s && member->type == MICROFEED_JSON_TYPE_DECIMAL) {
+			decimal = member->decimal;
+		}
+	}
+		
+	return decimal;
+}
+
 const char* microfeed_json_get_string(MicrofeedJson* json, const char* name) {
 	const char* string = NULL;
 	Member* member;
@@ -569,6 +658,42 @@ void microfeed_json_append_object(MicrofeedJson* json, MicrofeedJson* object) {
 	microfeed_store_insert(json->members, member);
 }
 
+void microfeed_json_set_integer(MicrofeedJson* json, const char* name, long long int integer) {
+	Member* member;
+
+	member = member_new(strdup(name), MICROFEED_JSON_TYPE_INTEGER);
+	member->integer = integer;
+	if ((member = microfeed_store_replace(json->members, member, Member))) {
+		member_free(member);
+	}
+}
+
+void microfeed_json_append_integer(MicrofeedJson* json, long long int integer) {
+	Member* member;
+
+	member = member_new(NULL, MICROFEED_JSON_TYPE_INTEGER);
+	member->integer = integer;
+	microfeed_store_insert(json->members, member);
+}
+
+void microfeed_json_set_decimal(MicrofeedJson* json, const char* name, double decimal) {
+	Member* member;
+
+	member = member_new(strdup(name), MICROFEED_JSON_TYPE_DECIMAL);
+	member->decimal = decimal;
+	if ((member = microfeed_store_replace(json->members, member, Member))) {
+		member_free(member);
+	}
+}
+
+void microfeed_json_append_decimal(MicrofeedJson* json, double decimal) {
+	Member* member;
+
+	member = member_new(NULL, MICROFEED_JSON_TYPE_DECIMAL);
+	member->decimal = decimal;
+	microfeed_store_insert(json->members, member);
+}
+
 void microfeed_json_set_string(MicrofeedJson* json, const char* name, const char* string) {
 	Member* member;
 
