@@ -39,15 +39,15 @@ struct _MicrofeedPublisher {
 	MicrofeedDatabaseEnvironment* database_environment;
 	char** image_properties;
 	MicrofeedProvider* provider;
-	DBusMessage* current_message;
 	
 	MicrofeedStore* subscribers;
 	MicrofeedStore* feeds;
 	MicrofeedFeed* settings_feed;
 	MicrofeedFeed* images_feed;
-	MicrofeedThread* singleton_update_thread;
 	MicrofeedThreadPool* thread_pool;
 	MicrofeedStore* timeouts;
+	MicrofeedMutex* mutex;
+	MicrofeedDatabase* timestamp_database;
 };
 
 typedef struct {
@@ -60,6 +60,7 @@ typedef struct {
 typedef struct {
 	MicrofeedFeed* feed;
 	void* implementation;
+	unsigned long int interval;
 } Timeout;
 
 typedef struct {
@@ -67,12 +68,21 @@ typedef struct {
 	MicrofeedError* (*callback)(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message);
 } MethodCallback;
 
+typedef struct {
+	MicrofeedPublisher* publisher;
+	Subscriber* subscriber;
+	DBusMessage* message;
+	MicrofeedError* (*callback)(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message);
+} MethodThreadData;
+
 static void object_unregister(DBusConnection* connection, void* user_data);
 static DBusHandlerResult object_message(DBusConnection* connection, DBusMessage* message, void* user_data);
+
 static Subscriber* subscriber_new(MicrofeedPublisher* publisher, const char* unique_connection_name);
 static void subscriber_free(Subscriber* subscriber);
 static const char* subscriber_get_unique_connection_name(Subscriber* subscriber);
-static Timeout* timeout_new(MicrofeedPublisher* publisher, MicrofeedFeed* feed);
+
+static Timeout* timeout_new(MicrofeedPublisher* publisher, MicrofeedFeed* feed, unsigned long int interval);
 static void timeout_free(Timeout* timeout);
 static MicrofeedFeed* timeout_get_feed(Timeout* timeout);
 
@@ -91,11 +101,15 @@ static MicrofeedError* method_update_feed(MicrofeedPublisher* publisher, Subscri
 static MicrofeedError* method_create_publisher(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message);
 static MicrofeedError* method_destroy_publisher(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message);
 
-static void reply_to_current_message(MicrofeedPublisher* publisher, MicrofeedError* error);
 static MicrofeedError* settings_feed_modify_item(MicrofeedFeed* feed, MicrofeedItem* existing_item, MicrofeedItem* new_item, void* user_data);
 static MicrofeedError* images_feed_download_item_data(MicrofeedFeed* feed, const char* uid, void** data, size_t* length, void* user_data);
+static int timeout_handler(void* data);
+static void* method_thread(void* data);
+static void* update_thread(void* data);
 static void thread_exited(MicrofeedThread* thread, void* user_data);
-static void timeout_handler(void* data);
+static MicrofeedFeed* get_or_instantiate_feed(MicrofeedPublisher* publisher, const char* uri);
+static void update_last_update_timestamp(MicrofeedFeed* feed);
+static void remove_data_from_store(void* data, void* user_data);
 
 static MethodCallback method_callbacks[] = {
 	{ MICROFEED_METHOD_NAME_ADD_ITEM, method_add_item },
@@ -184,14 +198,14 @@ MicrofeedPublisher* microfeed_publisher_new(const char* publisher_identifier, co
 
 		publisher->settings_feed = microfeed_feed_new(publisher, MICROFEED_FEED_URI_SETTINGS, "Settings", MICROFEED_FEED_PERMISSION_NONE, &settings_feed_callbacks, NULL);
 		microfeed_store_insert(publisher->feeds, publisher->settings_feed);
-		microfeed_feed_add_subscriber(publisher->settings_feed, NULL);		
 
 		publisher->images_feed = microfeed_feed_new(publisher, MICROFEED_FEED_URI_IMAGES, "Images", MICROFEED_FEED_PERMISSION_NONE, &images_feed_callbacks, NULL);
 		microfeed_store_insert(publisher->feeds, publisher->images_feed);
-		microfeed_feed_add_subscriber(publisher->images_feed, NULL);		
 
 		publisher->thread_pool = microfeed_thread_pool_new_with_exit_callback(microfeed_publisher_get_setting_value_integer(publisher, "threads.max", 10), thread_exited, publisher);
 		publisher->timeouts = microfeed_store_new_sorted(microfeed_store_compare_keys_direct, (MicrofeedStoreGetKeyFunction)timeout_get_feed);
+		publisher->mutex = microfeed_mutex_new();
+		publisher->timestamp_database = microfeed_database_environment_get_database(publisher->database_environment, ":microfeed.timestamps", NULL);
 	}
 	
 	return publisher;
@@ -204,22 +218,20 @@ MicrofeedPublisher* microfeed_publisher_new(const char* publisher_identifier, co
  */
 void microfeed_publisher_free(MicrofeedPublisher* publisher) {
 	dbus_connection_unregister_object_path(publisher->connection, publisher->object_path);
+
+	microfeed_feed_unref(publisher->settings_feed);
+	microfeed_feed_unref(publisher->images_feed);
 	microfeed_database_environment_free(publisher->database_environment);
-	microfeed_feed_remove_subscriber(publisher->settings_feed, NULL);
-	microfeed_feed_remove_subscriber(publisher->images_feed, NULL);
 
 	free(publisher->identifier);
 	free(publisher->object_path);
 	free(publisher->image_properties);
 	microfeed_store_free(publisher->subscribers);
 	microfeed_store_free(publisher->feeds);
-/* TODO
-	microfeed_feed_unref(publisher->settings_feed);
-	microfeed_feed_unref(publisher->images_feed);
-*/
-	microfeed_thread_free(publisher->singleton_update_thread);
+
 /* TODO	microfeed_thread_pool_free(publisher->thread_pool); */
 	microfeed_store_free(publisher->timeouts);
+	microfeed_mutex_free(publisher->mutex);
 }
 
 /**
@@ -336,24 +348,6 @@ int microfeed_publisher_set_setting_value(MicrofeedPublisher* publisher, const c
 	return retvalue;
 }
 
-MicrofeedThread* microfeed_publisher_queue_thread(MicrofeedPublisher* publisher, MicrofeedThreadFunction function, void* data) {
-	
-	return microfeed_thread_pool_queue_thread(publisher->thread_pool, function, data);
-}
-
-MicrofeedThread* microfeed_publisher_start_singleton_update_thread(MicrofeedPublisher* publisher, MicrofeedThreadFunction function, void* user_data) {
-	if (!publisher->singleton_update_thread) {
-		publisher->singleton_update_thread = microfeed_thread_new(function, user_data);		
-	}
-		
-	return publisher->singleton_update_thread;
-}
-
-MicrofeedThread* microfeed_publisher_get_singleton_update_thread(MicrofeedPublisher* publisher) {
-	
-	return publisher->singleton_update_thread;
-}
-
 MicrofeedDatabaseEnvironment* microfeed_publisher_get_database_environment(MicrofeedPublisher* publisher) {
 
 	return publisher->database_environment;
@@ -363,6 +357,8 @@ void microfeed_publisher_handle_item_property_change(MicrofeedPublisher* publish
 	char** key;
 	const char* old_value;
 	const char* new_value;	
+	
+	microfeed_mutex_lock(publisher->mutex);
 
 	for (key = publisher->image_properties; *key; key++) {
 		if (old_item && (old_value = microfeed_item_get_property(old_item, *key))) {
@@ -378,6 +374,8 @@ void microfeed_publisher_handle_item_property_change(MicrofeedPublisher* publish
 			microfeed_feed_ref_item_data(publisher->images_feed, new_value);
 		}
 	}
+
+	microfeed_mutex_unlock(publisher->mutex);
 }
 
 void microfeed_publisher_send_item_signal(MicrofeedPublisher* publisher, const char* destination, const char* signal_name, const char* uri, MicrofeedItem* item) {
@@ -389,6 +387,8 @@ void microfeed_publisher_send_item_signal(MicrofeedPublisher* publisher, const c
 	char status;
 	const char* key;
 	const char* value;
+
+	printf("%s : %s (%s)\n", publisher->identifier, signal_name, destination);
 	
 	message = dbus_message_new_signal(publisher->object_path, (destination ? MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION : MICROFEED_DBUS_INTERFACE_PUBLISHER), signal_name);
 	if (destination) {
@@ -416,6 +416,8 @@ void microfeed_publisher_send_item_signal(MicrofeedPublisher* publisher, const c
 void microfeed_publisher_send_item_uid_signal(MicrofeedPublisher* publisher, const char* destination, const char* signal_name, const char* uri, const char* uid) {
 	DBusMessage* message;
 
+	printf("%s : %s (%s)\n", publisher->identifier, signal_name, destination);
+
 	message = dbus_message_new_signal(publisher->object_path, (destination ? MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION : MICROFEED_DBUS_INTERFACE_PUBLISHER), signal_name);
 	if (destination) {
 		dbus_message_set_destination(message, destination);
@@ -428,6 +430,8 @@ void microfeed_publisher_send_item_uid_signal(MicrofeedPublisher* publisher, con
 void microfeed_publisher_send_status_changed_signal(MicrofeedPublisher* publisher, const char* destination, const char* uri, const char* uid, const char status) {
 	DBusMessage* message;
 
+	printf("%s : %s (%s)\n", publisher->identifier, MICROFEED_SIGNAL_NAME_ITEM_STATUS_CHANGED, destination);
+
 	message = dbus_message_new_signal(publisher->object_path, (destination ? MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION : MICROFEED_DBUS_INTERFACE_PUBLISHER), MICROFEED_SIGNAL_NAME_ITEM_STATUS_CHANGED);
 	if (destination) {
 		dbus_message_set_destination(message, destination);
@@ -440,6 +444,8 @@ void microfeed_publisher_send_status_changed_signal(MicrofeedPublisher* publishe
 void microfeed_publisher_send_feed_signal(MicrofeedPublisher* publisher, const char* destination, const char* signal_name, const char* uri) {
 	DBusMessage* message;
 
+	printf("%s : %s (%s)\n", publisher->identifier, signal_name, destination);
+
 	message = dbus_message_new_signal(publisher->object_path, (destination ? MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION : MICROFEED_DBUS_INTERFACE_PUBLISHER), signal_name);
 	if (destination) {
 		dbus_message_set_destination(message, destination);
@@ -454,6 +460,8 @@ void microfeed_publisher_send_error_signal(MicrofeedPublisher* publisher, const
 	DBusMessageIter iter;
 	const char* string;
 
+	printf("%s : %s (%s)\n", publisher->identifier, error_name, destination);
+
 	message = dbus_message_new_signal(publisher->object_path, (destination ? MICROFEED_DBUS_INTERFACE_ERROR_TO_DESTINATION : MICROFEED_DBUS_INTERFACE_ERROR), error_name);
 	if (destination) {
 		dbus_message_set_destination(message, destination);
@@ -474,6 +482,8 @@ void microfeed_publisher_send_item_data_signal(MicrofeedPublisher* publisher, co
 	DBusMessageIter iter;
 	DBusMessageIter subiter;
 
+	printf("%s : %s (%s)\n", publisher->identifier, MICROFEED_SIGNAL_NAME_ITEM_DATA, destination);
+
 	message = dbus_message_new_signal(publisher->object_path, (destination ? MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION : MICROFEED_DBUS_INTERFACE_PUBLISHER), MICROFEED_SIGNAL_NAME_ITEM_DATA);
 	if (destination) {
 		dbus_message_set_destination(message,destination);
@@ -489,10 +499,12 @@ void microfeed_publisher_send_item_data_signal(MicrofeedPublisher* publisher, co
 }
 
 const char* microfeed_publisher_get_object_path(MicrofeedPublisher* publisher) {
+
 	return publisher->object_path;
 }
 
 const char* microfeed_publisher_get_identifier(MicrofeedPublisher* publisher) {
+
 	return publisher->identifier;
 }
 
@@ -501,18 +513,15 @@ void microfeed_publisher_remove_subscriber(MicrofeedPublisher* publisher, const
 	MicrofeedStoreIterator* iterator;
 	MicrofeedFeed* feed;
 	char buffer[1024];
+
+	microfeed_mutex_lock(publisher->mutex);
 	
 	if ((subscriber = microfeed_store_get(publisher->subscribers, unique_connection_name, Subscriber))) {
 		for (iterator = microfeed_store_iterate(subscriber->feeds, NULL);
 		     (feed = (MicrofeedFeed*)microfeed_store_iterator_get(iterator, MicrofeedFeed));
 		     microfeed_store_iterator_next(iterator)) {
-			microfeed_feed_remove_subscriber(feed, unique_connection_name);
 			microfeed_store_remove(subscriber->feeds, feed);
-	
-			if (microfeed_feed_get_subscriber_count(feed) == 0) {
-				microfeed_store_remove(publisher->feeds, feed);
-				microfeed_feed_unref(feed);
-			}
+			microfeed_feed_unref(feed);
 		}
 
 		snprintf(buffer, 1024, "type='signal',sender='org.freedesktop.DBus',interface='org.freedesktop.DBus',method='NameOwnerChanged',arg0='%s'", unique_connection_name);
@@ -527,6 +536,8 @@ void microfeed_publisher_remove_subscriber(MicrofeedPublisher* publisher, const
 			publisher->callbacks.no_more_subscribers(publisher, publisher->user_data);
 		}
 	}			
+
+	microfeed_mutex_unlock(publisher->mutex);
 }
 
 MicrofeedProvider* microfeed_publisher_get_provider(MicrofeedPublisher* publisher) {
@@ -563,15 +574,13 @@ static const char* subscriber_get_unique_connection_name(Subscriber* subscriber)
 	return subscriber->unique_connection_name;
 }
 
-static Timeout* timeout_new(MicrofeedPublisher* publisher, MicrofeedFeed* feed) {
+static Timeout* timeout_new(MicrofeedPublisher* publisher, MicrofeedFeed* feed, unsigned long int interval) {
 	Timeout* timeout = NULL;
-	unsigned long int interval;
 	
-	if ((interval = (unsigned long int)microfeed_publisher_get_setting_value_integer(publisher, "update_interval", 0))) {
-		timeout = microfeed_memory_allocate(Timeout);
-		timeout->feed = feed;
-		timeout->implementation = publisher->callbacks.add_timeout(publisher, interval * 60000, timeout_handler, timeout, publisher->user_data);
-	}
+	timeout = microfeed_memory_allocate(Timeout);
+	timeout->feed = feed;
+	timeout->interval = interval;
+	timeout->implementation = publisher->callbacks.add_timeout(publisher, (timeout->interval ? timeout->interval * 60000 : 1000), timeout_handler, timeout, publisher->user_data);
 	
 	return timeout;
 }
@@ -595,6 +604,7 @@ static MicrofeedError* method_add_item(MicrofeedPublisher* publisher, Subscriber
 	MicrofeedItem* item;
 	const char* key;
 	const char* value;
+	MicrofeedError* error;
 	
 	if (!dbus_message_iter_init(message, &iter) || dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_STRING) {
 		
@@ -606,7 +616,7 @@ static MicrofeedError* method_add_item(MicrofeedPublisher* publisher, Subscriber
 		return microfeed_error_new(MICROFEED_ERROR_INVALID_ARGUMENTS, "Expected temporary item uid argument in AddItem method.");
 	}
 	dbus_message_iter_get_basic(&iter, &uid);
-	if (!(feed = microfeed_store_get(subscriber->feeds, uri, MicrofeedFeed))) {
+	if (!(feed = get_or_instantiate_feed(publisher, uri))) {
 		
 		return microfeed_error_new(MICROFEED_ERROR_NO_SUCH_FEED, "Trying to add an item into a feed that does not exist.");
 	}
@@ -635,12 +645,12 @@ static MicrofeedError* method_add_item(MicrofeedPublisher* publisher, Subscriber
 		microfeed_item_set_property(item, key, value);
 	}
 	
-	reply_to_current_message(publisher, NULL);
-	
-	microfeed_feed_call_modify_item_callback(feed, NULL, item);
+	error = microfeed_feed_call_modify_item_callback(feed, NULL, item);
 	microfeed_item_free(item);
+
+	microfeed_feed_unref(feed);
 	
-	return NULL;
+	return error;
 }
 
 static MicrofeedError* method_modify_item(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message) {
@@ -652,6 +662,7 @@ static MicrofeedError* method_modify_item(MicrofeedPublisher* publisher, Subscri
 	MicrofeedItem* new_item;
 	const char* key;
 	const char* value;
+	MicrofeedError* error;
 	
 	if (!dbus_message_iter_init(message, &iter) || dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_STRING) {
 		
@@ -663,7 +674,7 @@ static MicrofeedError* method_modify_item(MicrofeedPublisher* publisher, Subscri
 		return microfeed_error_new(MICROFEED_ERROR_INVALID_ARGUMENTS, "Expected item uid argument in ModifyItem method.");
 	}
 	dbus_message_iter_get_basic(&iter, &uid);
-	if (!(feed = microfeed_store_get(subscriber->feeds, uri, MicrofeedFeed))) {
+	if (!(feed = get_or_instantiate_feed(publisher, uri))) {
 		
 		return microfeed_error_new(MICROFEED_ERROR_NO_SUCH_FEED, "Trying to modify an item of a feed that does not exist.");
 	}
@@ -692,16 +703,16 @@ static MicrofeedError* method_modify_item(MicrofeedPublisher* publisher, Subscri
 		microfeed_item_set_property(new_item, key, value);
 	}
 	
-	reply_to_current_message(publisher, NULL);
-	
-	microfeed_feed_call_modify_item_callback(feed, existing_item, new_item);
+	error = microfeed_feed_call_modify_item_callback(feed, existing_item, new_item);
 	microfeed_item_free(new_item);
 	
-	return NULL;
+	microfeed_feed_unref(feed);
+
+	return error;
 }
 
 static MicrofeedError* method_remove_item(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message) {
-	MicrofeedError* error = NULL;
+	MicrofeedError* error;
 	DBusError dbus_error;
 	const char* uri;
 	const char* uid;
@@ -711,21 +722,22 @@ static MicrofeedError* method_remove_item(MicrofeedPublisher* publisher, Subscri
 	dbus_error_init(&dbus_error);
 	if (!dbus_message_get_args(message, &dbus_error, DBUS_TYPE_STRING, &uri, DBUS_TYPE_STRING, &uid, DBUS_TYPE_INVALID)) {
 		error = microfeed_error_new(MICROFEED_ERROR_INVALID_ARGUMENTS, "Expected feed uri and item uid arguments in RemoveItem method");
-	} else if (!(feed = microfeed_store_get(subscriber->feeds, uri, MicrofeedFeed))) {
+	} else if (!(feed = get_or_instantiate_feed(publisher, uri))) {
 		error = microfeed_error_new(MICROFEED_ERROR_NO_SUCH_FEED, "Trying to remove an item of a feed that does not exist.");
 	} else if (!(item = microfeed_feed_get_item(feed, uid))) {
 		error = microfeed_error_new(MICROFEED_ERROR_NO_SUCH_ITEM, "Trying to remove an item that does not exist.");
 	} else {
-		reply_to_current_message(publisher, NULL);
-	
-		microfeed_feed_call_modify_item_callback(feed, item, NULL);
+		error = microfeed_feed_call_modify_item_callback(feed, item, NULL);
 		microfeed_item_free(item);
+
+		microfeed_feed_unref(feed);
 	}
 	
 	return error;
 }
 
 static MicrofeedError* method_read_items(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message) {
+	MicrofeedError* error;
 	DBusMessageIter iter;
 	const char* uri;
 	const char* start_uid;
@@ -747,16 +759,16 @@ static MicrofeedError* method_read_items(MicrofeedPublisher* publisher, Subscrib
 		return microfeed_error_new(MICROFEED_ERROR_INVALID_ARGUMENTS, "Expected end item uid argument in ReadItem method.");
 	}
 	dbus_message_iter_get_basic(&iter, &end_uid);
-	if (!(feed = microfeed_store_get(subscriber->feeds, uri, MicrofeedFeed))) {
+	if (!(feed = get_or_instantiate_feed(publisher, uri))) {
 		
 		return microfeed_error_new(MICROFEED_ERROR_NO_SUCH_FEED, "Trying to read items of a feed that does not exist.");
 	}
 	
-	reply_to_current_message(publisher, NULL);
+	error = microfeed_feed_unset_item_statuses(feed, start_uid, end_uid, MICROFEED_ITEM_STATUS_UNREAD);
 
-	microfeed_feed_unset_item_statuses(feed, start_uid, end_uid, MICROFEED_ITEM_STATUS_UNREAD);
+	microfeed_feed_unref(feed);
 
-	return NULL;
+	return error;
 }
 
 static MicrofeedError* method_republish_items(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message) {
@@ -790,21 +802,23 @@ static MicrofeedError* method_republish_items(MicrofeedPublisher* publisher, Sub
 
 	if (!(feed = microfeed_store_get(subscriber->feeds, uri, MicrofeedFeed))) {
 
-		return microfeed_error_new(MICROFEED_ERROR_NO_SUCH_FEED, "Trying to republish items of a feed that does not exist.");
+		return microfeed_error_new(MICROFEED_ERROR_NO_SUCH_FEED, "Trying to republish items of a feed that is not subscribed.");
 	}
 	
-	reply_to_current_message(publisher, NULL);
-	
 	microfeed_feed_republish(feed, (*start_uid ? start_uid : NULL), (*end_uid ? end_uid : NULL), max_count, subscriber->unique_connection_name);
-	     
+	
 	return NULL;
 }
 
 static MicrofeedError* method_send_item_data(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message) {
+	MicrofeedError* error;
 	DBusMessageIter iter;
 	const char* uri;
 	const char* uid;
 	MicrofeedFeed* feed;
+	void* data;
+	size_t data_size;
+	DBusMessage* reply;
 
 	if (!publisher->callbacks.download_image) {
 
@@ -812,7 +826,7 @@ static MicrofeedError* method_send_item_data(MicrofeedPublisher* publisher, Subs
 	}
 	if (!dbus_message_iter_init(message, &iter) || dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_STRING) {
 		
-		return microfeed_error_new(MICROFEED_ERROR_INVALID_ARGUMENTS, "Expected a feed uri argument in SendItemData method");
+		return microfeed_error_new(MICROFEED_ERROR_INVALID_ARGUMENTS, "Expected a feed uri argument in GetItemData method");
 	}
 	dbus_message_iter_get_basic(&iter, &uri);
 	if (!dbus_message_iter_next(&iter) || dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_STRING) {
@@ -823,14 +837,12 @@ static MicrofeedError* method_send_item_data(MicrofeedPublisher* publisher, Subs
 	
 	if (!(feed = microfeed_store_get(subscriber->feeds, uri, MicrofeedFeed))) {
 
-		return microfeed_error_new(MICROFEED_ERROR_NO_SUCH_FEED, "Trying to republish items of a feed that does not exist.");
+		return microfeed_error_new(MICROFEED_ERROR_NO_SUCH_FEED, "Trying to get item data of a feed that is not subscribed.");
 	}
 	
-	reply_to_current_message(publisher, NULL);
-	
-	microfeed_feed_send_item_data(feed, uid, subscriber->unique_connection_name);
-		
-	return NULL;
+	error = microfeed_feed_send_item_data(feed, uid, subscriber->unique_connection_name);
+
+	return error;
 }
 
 static MicrofeedError* method_mark_item(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message) {
@@ -843,12 +855,12 @@ static MicrofeedError* method_mark_item(MicrofeedPublisher* publisher, Subscribe
 	dbus_error_init(&dbus_error);
 	if (!dbus_message_get_args(message, &dbus_error, DBUS_TYPE_STRING, &uri, DBUS_TYPE_STRING, &uid, DBUS_TYPE_INVALID)) {
 		error = microfeed_error_new(MICROFEED_ERROR_INVALID_ARGUMENTS, "Expected feed uri and item uid arguments in MarkItem method");
-	} else if (!(feed = microfeed_store_get(subscriber->feeds, uri, MicrofeedFeed))) {
+	} else if (!(feed = get_or_instantiate_feed(publisher, uri))) {
 		error = microfeed_error_new(MICROFEED_ERROR_NO_SUCH_FEED, "Trying to mark an item of a feed that does not exist.");
 	} else {
-		reply_to_current_message(publisher, NULL);
-		
-		microfeed_feed_set_item_status(feed, uid, MICROFEED_ITEM_STATUS_MARKED);
+		error = microfeed_feed_set_item_status(feed, uid, MICROFEED_ITEM_STATUS_MARKED);
+
+		microfeed_feed_unref(feed);
 	}
 
 	return error;
@@ -864,12 +876,12 @@ static MicrofeedError* method_unmark_item(MicrofeedPublisher* publisher, Subscri
 	dbus_error_init(&dbus_error);
 	if (!dbus_message_get_args(message, &dbus_error, DBUS_TYPE_STRING, &uri, DBUS_TYPE_STRING, &uid, DBUS_TYPE_INVALID)) {
 		error = microfeed_error_new(MICROFEED_ERROR_INVALID_ARGUMENTS, "Expected feed uri and item uid arguments in UnarkItem method");
-	} else if (!(feed = microfeed_store_get(subscriber->feeds, uri, MicrofeedFeed))) {
+	} else if (!(feed = get_or_instantiate_feed(publisher, uri))) {
 		error = microfeed_error_new(MICROFEED_ERROR_NO_SUCH_FEED, "Trying to unmark an item of a feed that does not exist.");
 	} else {
-		reply_to_current_message(publisher, NULL);
-		
-		microfeed_feed_unset_item_status(feed, uid, MICROFEED_ITEM_STATUS_MARKED);
+		error = microfeed_feed_unset_item_status(feed, uid, MICROFEED_ITEM_STATUS_MARKED);
+
+		microfeed_feed_unref(feed);
 	}
 
 	return error;
@@ -884,8 +896,13 @@ static MicrofeedError* method_subscribe_feed(MicrofeedPublisher* publisher, Subs
 	MicrofeedFeed* feed;
 	DBusError error;
 	char* uri;
+	DBusMessage* reply;
 	Timeout* timeout;
-	int new_feed = 0;
+	unsigned long int interval;
+	time_t timestamp = 0;
+	void* data;
+	size_t data_size;	
+	time_t elapsed;
 
 	dbus_error_init(&error);
 	if (!dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &uri, DBUS_TYPE_INVALID)) {
@@ -893,36 +910,51 @@ static MicrofeedError* method_subscribe_feed(MicrofeedPublisher* publisher, Subs
 		return microfeed_error_new(MICROFEED_ERROR_INVALID_ARGUMENTS, "Expected feed uri argument in SubscribeFeed method");
 	}
 	
-	if (!(feed = microfeed_store_get(publisher->feeds, uri, MicrofeedFeed))) {
-		new_feed = 1;
-		if ((feed = publisher->callbacks.instantiate_feed(publisher, uri, publisher->user_data))) {
-			microfeed_store_insert(publisher->feeds, feed);
-			if ((timeout = timeout_new(publisher, feed))) {
-				microfeed_store_insert(publisher->timeouts, timeout);
-			}
-		} else {
+	if (microfeed_store_get(subscriber->feeds, uri, MicrofeedFeed)) {
 
-			return microfeed_error_new(MICROFEED_ERROR_NO_SUCH_FEED, "Failed to subscribe a new feed");
-		}
+		return microfeed_error_new(MICROFEED_ERROR_FEED_ALREADY_SUBSCRIBED, "Cannot subscribe the feed twice.");
 	}
+
+	if (!(feed = get_or_instantiate_feed(publisher, uri))) {
+	
+		return microfeed_error_new(MICROFEED_ERROR_NO_SUCH_FEED, "Cannot subscribe the feed.");
+	}
+	
 	microfeed_store_insert(subscriber->feeds, feed);
 
-	reply_to_current_message(publisher, NULL);
+	printf("%s > OK (early sending)\n", publisher->identifier);
+	reply = dbus_message_new_method_return(message);
+	dbus_connection_send(publisher->connection, reply, NULL);
+	dbus_message_unref(reply);
 
-	microfeed_feed_add_subscriber(feed, subscriber->unique_connection_name);
 	microfeed_publisher_send_item_signal(publisher, subscriber->unique_connection_name, MICROFEED_SIGNAL_NAME_ITEM_ADDED, uri, microfeed_feed_get_metadata_item(feed));
-	if (new_feed) {
-		microfeed_feed_update(feed, NULL);
+
+	if ((interval = (unsigned long int)microfeed_publisher_get_setting_value_integer(publisher, "update_interval", 0))) {
+		if ((microfeed_database_get_data(publisher->timestamp_database, &uri, strlen(uri) + 1, &data, &data_size))) {
+			if (data_size >= sizeof(time_t)) {
+				timestamp = *((time_t*)data);
+			}
+			free(data);
+		}
+
+printf("LAST UPDATE: %ld\n", timestamp);
+
+		elapsed = time(NULL) - timestamp;
+		if (elapsed > interval) {
+			timeout = timeout_new(publisher, feed, 0);
+		} else {
+			timeout = timeout_new(publisher, feed, interval - elapsed);
+		}
+		microfeed_store_insert(publisher->timeouts, timeout);
 	}
 
-	return NULL;
+	return (MicrofeedError*)-1;
 }
 
 static MicrofeedError* method_unsubscribe_feed(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message) {
 	DBusError error;
 	char* uri;
 	MicrofeedFeed* feed;
-	Timeout* timeout;
 	
 	dbus_error_init(&error);
 	if (!dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &uri, DBUS_TYPE_INVALID)) {
@@ -934,16 +966,7 @@ static MicrofeedError* method_unsubscribe_feed(MicrofeedPublisher* publisher, Su
 		return microfeed_error_new(MICROFEED_ERROR_FEED_NOT_SUBSCRIBED, "Trying to unsubscribe a feed that is not subscribed");
 	}
 
-	reply_to_current_message(publisher, NULL);
-	
-	microfeed_feed_remove_subscriber(feed, subscriber->unique_connection_name);
-	if (microfeed_feed_get_subscriber_count(feed) == 0) {
-		if ((timeout = microfeed_store_get(publisher->timeouts, feed, Timeout))) {
-			timeout_free(timeout);
-		}
-		microfeed_store_remove(publisher->feeds, feed);
-		microfeed_feed_unref(feed);
-	}
+	microfeed_feed_unref(feed);
 
 	return NULL;
 }
@@ -959,14 +982,15 @@ static MicrofeedError* method_update_feed(MicrofeedPublisher* publisher, Subscri
 		return microfeed_error_new(MICROFEED_ERROR_INVALID_ARGUMENTS, "Expected feed uri argument in UpdateFeed method");
 	}
 	
-	if (!(feed = microfeed_store_get(subscriber->feeds, uri, MicrofeedFeed))) {
+	if (!(feed = get_or_instantiate_feed(publisher, uri))) {
 	
 		return microfeed_error_new(MICROFEED_ERROR_FEED_NOT_SUBSCRIBED, "Trying to update a feed that is not subscribed");
 	}
 
-	reply_to_current_message(publisher, NULL);
-	
 	microfeed_feed_update(feed, subscriber->unique_connection_name);
+	update_last_update_timestamp(feed);
+
+	microfeed_feed_unref(feed);
 	
 	return NULL;
 }
@@ -974,8 +998,6 @@ static MicrofeedError* method_update_feed(MicrofeedPublisher* publisher, Subscri
 static MicrofeedError* method_create_publisher(MicrofeedPublisher* publisher, Subscriber* subscriber, DBusMessage* message) {
 	/* TODO: Check that publisher does not exist. */
 
-	reply_to_current_message(publisher, NULL);
-	
 	microfeed_publisher_add_setting(publisher, "archive_old_items", "Archive old items", "hours", "non-negative integer", "5", "5");
 	microfeed_publisher_add_setting(publisher, "update_interval", "Update interval", "minutes", "non-negative integer", "5", "1");
 	
@@ -990,8 +1012,8 @@ static MicrofeedError* method_destroy_publisher(MicrofeedPublisher* publisher, S
 	MicrofeedStoreIterator* feed_iterator;
 	MicrofeedFeed* feed;
 
-	reply_to_current_message(publisher, NULL);
-	
+	microfeed_mutex_lock(publisher->mutex);
+
 	dbus_connection_unregister_object_path(publisher->connection, publisher->object_path);
 	
 	for (subscriber_iterator = microfeed_store_iterate(publisher->subscribers, NULL);
@@ -1000,16 +1022,18 @@ static MicrofeedError* method_destroy_publisher(MicrofeedPublisher* publisher, S
 		for (feed_iterator = microfeed_store_iterate(feed_subscriber->feeds, NULL);
 		     (feed = microfeed_store_iterator_get(feed_iterator, MicrofeedFeed));
 		     microfeed_store_iterator_next(feed_iterator)) {
-			microfeed_feed_remove_subscriber(feed, subscriber->unique_connection_name);
+			microfeed_feed_unref(feed);
 			microfeed_publisher_send_error_signal(publisher, subscriber->unique_connection_name, MICROFEED_ERROR_NO_SUCH_PUBLISHER, NULL, NULL, "Publisher destroyed.");
 		}
 	}
 	for (feed_iterator = microfeed_store_iterate(publisher->feeds, NULL);
 	     (feed = microfeed_store_iterator_get(feed_iterator, MicrofeedFeed));
 	     microfeed_store_iterator_next(feed_iterator)) {
-		microfeed_feed_unref(feed);
+		microfeed_feed_remove_free_callback(feed, remove_data_from_store, publisher->feeds);
 	}
 
+	microfeed_mutex_unlock(publisher->mutex);
+
 	return NULL;	
 }
 
@@ -1024,8 +1048,12 @@ static DBusHandlerResult object_message(DBusConnection* connection, DBusMessage*
 	Subscriber* subscriber;
 	char buffer[1024];
 	MicrofeedError* error;
+	MethodThreadData* mtd;
 
 	publisher = (MicrofeedPublisher*)user_data;	
+	
+	microfeed_mutex_lock(publisher->mutex);
+
 	unique_connection_name = dbus_message_get_sender(message);
 	if (!(subscriber = (Subscriber*)microfeed_store_get(publisher->subscribers, unique_connection_name, Subscriber))) {
 		subscriber = subscriber_new(publisher, unique_connection_name);
@@ -1038,19 +1066,21 @@ static DBusHandlerResult object_message(DBusConnection* connection, DBusMessage*
 	subscriber->last_activity = time(NULL);
 	for (i = 0; method_callbacks[i].name; i++) {
 		if (dbus_message_is_method_call(message, MICROFEED_DBUS_INTERFACE_PUBLISHER, method_callbacks[i].name)) {
-			printf("%s (%s)\n", method_callbacks[i].name, unique_connection_name);
-			publisher->current_message = message;
-			error = method_callbacks[i].callback(publisher, subscriber, message);
-			reply_to_current_message(publisher, error);
-			if (error) {
-				microfeed_error_free(error);
-			}
+			printf("%s < %s (%s)\n", publisher->identifier, method_callbacks[i].name, unique_connection_name);
+
+			mtd = microfeed_memory_allocate(MethodThreadData);
+			mtd->publisher = publisher;
+			mtd->subscriber = subscriber;
+			mtd->callback = method_callbacks[i].callback;
+			mtd->message = dbus_message_ref(message);
+			microfeed_thread_pool_queue_thread_with_exit_callback(publisher->thread_pool, method_thread, mtd, thread_exited, publisher);
+
 			result = DBUS_HANDLER_RESULT_HANDLED;
 			break;
 		}
 	}
 	
-	if (microfeed_store_get_size(subscriber->feeds) == 0) {
+	if (result != DBUS_HANDLER_RESULT_HANDLED && microfeed_store_get_size(subscriber->feeds) == 0) {
 		snprintf(buffer, 1024, "type='signal',sender='org.freedesktop.DBus',interface='org.freedesktop.DBus',method='NameOwnerChanged',arg0='%s'", unique_connection_name);
 		dbus_bus_remove_match(connection, buffer, NULL);
 
@@ -1058,32 +1088,11 @@ static DBusHandlerResult object_message(DBusConnection* connection, DBusMessage*
 		subscriber_free(subscriber);
 	}
 
-	if (publisher->callbacks.no_more_subscribers &&
-	    microfeed_store_get_size(publisher->subscribers) == 0 &&
-	    microfeed_thread_pool_get_started_thread_count(publisher->thread_pool) == 0) {
-		publisher->callbacks.no_more_subscribers(publisher, publisher->user_data);
-	}
+	microfeed_mutex_unlock(publisher->mutex);
 
 	return result;
 }
 
-static void reply_to_current_message(MicrofeedPublisher* publisher, MicrofeedError* error) {
-	char buffer[1024];
-	DBusMessage* reply;
-
-	if (publisher->current_message) {
-		if (error) {
-			snprintf(buffer, 1024, "%s.%s", MICROFEED_DBUS_INTERFACE_ERROR, microfeed_error_get_name(error));
-			reply = dbus_message_new_error(publisher->current_message, buffer, microfeed_error_get_message(error));
-		} else {
-			reply = dbus_message_new_method_return(publisher->current_message);
-		}
-		dbus_connection_send(publisher->connection, reply, NULL);
-		dbus_message_unref(reply);
-		publisher->current_message = NULL;
-	}	
-}
-
 static MicrofeedError* settings_feed_modify_item(MicrofeedFeed* feed, MicrofeedItem* existing_item, MicrofeedItem* new_item, void* user_data) {
 	MicrofeedError* error = NULL;;
 	MicrofeedPublisher* publisher;
@@ -1120,20 +1129,144 @@ static MicrofeedError* images_feed_download_item_data(MicrofeedFeed* feed, const
 	return error;
 }
 
+static int timeout_handler(void* data) {
+	int retvalue = 1;
+	Timeout* timeout;
+	MicrofeedPublisher* publisher;
+	MicrofeedFeed* feed;
+	unsigned long int interval;
+	
+	timeout = (Timeout*)data;
+	feed = timeout->feed;
+	publisher = microfeed_feed_get_publisher(feed);
+
+	interval = (unsigned long int)microfeed_publisher_get_setting_value_integer(publisher, "update_interval", 0);
+	if (interval && interval != timeout->interval) {
+		retvalue = 0;
+		microfeed_store_remove(publisher->timeouts, timeout);
+
+	printf("INTERVAL: %ld\n", interval);
+		if (interval) {
+			timeout = timeout_new(publisher, feed, interval);
+			microfeed_store_insert(publisher->timeouts, timeout);
+		}
+	}
+
+	microfeed_thread_pool_queue_thread_with_exit_callback(publisher->thread_pool, update_thread, feed, thread_exited, publisher);
+
+	return retvalue;
+}
+
+static void* method_thread(void* data) {
+	MethodThreadData* mtd;
+	MicrofeedError* error;
+	char buffer[1024];
+	DBusMessage* reply;
+	
+	mtd = (MethodThreadData*)data;
+
+	error = mtd->callback(mtd->publisher, mtd->subscriber, mtd->message);
+
+	microfeed_mutex_lock(mtd->publisher->mutex);
+
+	if (!error) {
+		printf("%s > OK\n", mtd->publisher->identifier);
+		reply = dbus_message_new_method_return(mtd->message);
+	} else if (error == (MicrofeedError*)-1) {
+		printf("%s > OK (sent already)\n", mtd->publisher->identifier);
+		reply = NULL;
+	} else {
+		printf("%s > %s (%s)\n", mtd->publisher->identifier, microfeed_error_get_name(error), microfeed_error_get_message(error));
+		snprintf(buffer, 1024, "%s.%s", MICROFEED_DBUS_INTERFACE_ERROR, microfeed_error_get_name(error));
+		reply = dbus_message_new_error(mtd->message, buffer, microfeed_error_get_message(error));
+		microfeed_error_free(error);
+	}
+	if (reply) {
+		dbus_connection_send(mtd->publisher->connection, reply, NULL);
+		dbus_message_unref(reply);
+	}
+	
+	if (microfeed_store_get_size(mtd->subscriber->feeds) == 0) {
+		snprintf(buffer, 1024, "type='signal',sender='org.freedesktop.DBus',interface='org.freedesktop.DBus',method='NameOwnerChanged',arg0='%s'", dbus_message_get_sender(mtd->message));
+		dbus_bus_remove_match(mtd->publisher->connection, buffer, NULL);
+
+		microfeed_store_remove(mtd->publisher->subscribers, mtd->subscriber);
+		subscriber_free(mtd->subscriber);
+	}
+	
+	microfeed_mutex_unlock(mtd->publisher->mutex);
+
+	dbus_message_unref(mtd->message);
+	microfeed_memory_free(mtd);
+
+	return NULL;
+}
+
+static void* update_thread(void* data) {
+	MicrofeedFeed* feed;
+	
+	feed = (MicrofeedFeed*)data;
+
+	microfeed_feed_update(feed, NULL);
+	update_last_update_timestamp(feed);
+	
+	return NULL;
+}
+
 static void thread_exited(MicrofeedThread* thread, void* user_data) {
 	MicrofeedPublisher* publisher;
 	
 	publisher = (MicrofeedPublisher*)user_data;
+
+	microfeed_mutex_lock(publisher->mutex);
+	
 	if (publisher->callbacks.no_more_subscribers &&
 	    microfeed_store_get_size(publisher->subscribers) == 0 &&
 	    microfeed_thread_pool_get_started_thread_count(publisher->thread_pool) == 0) {
 		publisher->callbacks.no_more_subscribers(publisher, publisher->user_data);
 	}
+
+	microfeed_mutex_unlock(publisher->mutex);
 }	
 
-static void timeout_handler(void* data) {
-	Timeout* timeout;
+static MicrofeedFeed* get_or_instantiate_feed(MicrofeedPublisher* publisher, const char* uri) {
+	MicrofeedFeed* feed;
+	
+	microfeed_mutex_lock(publisher->mutex);
 
-	timeout = (Timeout*)data;
-	microfeed_feed_update(timeout->feed, NULL);
+	if (!(feed = microfeed_store_get(publisher->feeds, uri, MicrofeedFeed))) {
+		if ((feed = publisher->callbacks.instantiate_feed(publisher, uri, publisher->user_data))) {
+			microfeed_store_insert(publisher->feeds, feed);
+			microfeed_feed_add_free_callback(feed, remove_data_from_store, publisher->feeds);
+		}
+	} else {
+		microfeed_feed_ref(feed);
+	}
+
+	microfeed_mutex_unlock(publisher->mutex);
+	
+	return feed;
+}
+
+static void update_last_update_timestamp(MicrofeedFeed* feed) {
+	MicrofeedPublisher* publisher;
+	time_t timestamp;
+	const char* uri;
+
+	publisher = microfeed_feed_get_publisher(feed);
+	timestamp = time(NULL);
+	uri = microfeed_feed_get_uri(feed);
+
+	microfeed_mutex_lock(publisher->mutex);
+
+	microfeed_database_replace_data(publisher->timestamp_database, uri, strlen(uri) + 1, &timestamp, sizeof(time_t));
+
+	microfeed_mutex_unlock(publisher->mutex);
+}
+
+static void remove_data_from_store(void* data, void* user_data) {
+	MicrofeedStore* store;
+	
+	store = (MicrofeedStore*)user_data;
+	microfeed_store_remove(store, data);
 }
