@@ -90,7 +90,7 @@ typedef int (*MicrofeedPublisherUpdateSettingCallback)(MicrofeedPublisher* publi
 
 typedef MicrofeedError* (*MicrofeedPublisherDownloadImageCallback)(MicrofeedPublisher* publisher, const char* uid, void** data, size_t* length, void* user_data);
 
-typedef void (*MicrofeedPublisherTimeoutHandler)(void* data);
+typedef int (*MicrofeedPublisherTimeoutHandler)(void* data);
 
 typedef void* (*MicrofeedPublisherAddTimeoutCallback)(MicrofeedPublisher* publisher, unsigned long int interval, MicrofeedPublisherTimeoutHandler handler, void* data, void* user_data);
 
@@ -130,9 +130,6 @@ void microfeed_publisher_remove_setting(MicrofeedPublisher* publisher, const cha
 char* microfeed_publisher_get_setting_value(MicrofeedPublisher* publisher, const char* uid, const char* default_value);
 long int microfeed_publisher_get_setting_value_integer(MicrofeedPublisher* publisher, const char* uid, long int default_value);
 int microfeed_publisher_set_setting_value(MicrofeedPublisher* publisher, const char* uid, const char* value);
-MicrofeedThread* microfeed_publisher_queue_thread(MicrofeedPublisher* publisher, MicrofeedThreadFunction function, void* data);
-MicrofeedThread* microfeed_publisher_start_singleton_update_thread(MicrofeedPublisher* publisher, MicrofeedThreadFunction function, void* user_data);
-MicrofeedThread* microfeed_publisher_get_singleton_update_thread(MicrofeedPublisher* publisher);
 
 /**
  * @}
