@@ -58,15 +58,23 @@ typedef struct {
 	MicrofeedStore* feeds;
 } Publisher;
 
+typedef struct _Subscription Subscription;
+struct _Subscription {
+	Subscription* next;
+	Subscription* previous;
+	MicrofeedSubscriberCallbacks* callbacks;
+	void* user_data;
+};
+
 typedef struct {
 	unsigned int reference_count;
 	Publisher* publisher;
 	char* uri;
 	char* name;
-	MicrofeedSubscriberCallbacks callbacks;
-	void* user_data;
+	Subscription* subscriptions;
 } Feed;
 
+
 typedef struct {
 	const char* name;
 	void (*callback)(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message);
@@ -90,14 +98,16 @@ static void provider_set_unique_connection_name(Provider* provider, const char*
 
 static Publisher* publisher_new(Provider* provider, const char* identifier);
 static void publisher_free(Publisher* publisher);
-Publisher* publisher_ref(Publisher* publisher);
-void publisher_unref(Publisher* publisher);
+static Publisher* publisher_ref(Publisher* publisher);
+static void publisher_unref(Publisher* publisher);
 static const char* publisher_get_identifier(Publisher* publisher);
 static const char* publisher_get_object_path(Publisher* publisher);
 
-static Feed* feed_new(Publisher* publisher, const char* uri, MicrofeedSubscriberCallbacks* callbacks, void* user_data);
+static Feed* feed_new(Publisher* publisher, const char* uri);
 static void feed_free(Feed* feed);
 static const char* feed_get_uri(Feed* feed);
+static void feed_add_subscription(Feed* feed, MicrofeedSubscriberCallbacks* callbacks, void* user_data);
+static void feed_remove_subscription(Feed* feed, MicrofeedSubscriberCallbacks* callbacks, void* user_data);
 
 static void call_publisher_method(Publisher* publisher, const char* uri, const char* uid, DBusMessage* message, MicrofeedSubscriberErrorCallback callback, void* user_data);
 static MicrofeedItem* parse_item_from_message(DBusMessageIter* iter);
@@ -138,13 +148,17 @@ static DBusObjectPathVTable object_vtable = {
 	object_message
 };	
 
-MicrofeedSubscriber* microfeed_subscriber_new(const char* identifier, const char* object_path, DBusConnection* connection) {
+static unsigned int subscriber_count = 0;
+
+MicrofeedSubscriber* microfeed_subscriber_new(const char* identifier, DBusConnection* connection) {
 	MicrofeedSubscriber* subscriber;
+	char buffer[1024];
 	
 	subscriber = microfeed_memory_allocate(MicrofeedSubscriber);
 	subscriber->identifier = strdup(identifier);
 	subscriber->connection = connection;
-	subscriber->object_path = strdup(object_path);
+	snprintf(buffer, 1024, "%s%u", MICROFEED_DBUS_OBJECT_PATH_PREFIX_SUBSCRIBER, subscriber_count++);
+	subscriber->object_path = strdup(buffer);
 	subscriber->configuration = microfeed_configuration_new();
 	subscriber->max_retries = 3;
 	
@@ -163,6 +177,11 @@ MicrofeedSubscriber* microfeed_subscriber_new(const char* identifier, const char
 
 		return NULL;
 	}
+
+	snprintf(buffer, 1024, "type='signal',interface='%s',destination='%s'", MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION, dbus_bus_get_unique_name(subscriber->connection));
+	dbus_bus_add_match(subscriber->connection, buffer, NULL);
+	snprintf(buffer, 1024, "type='signal',interface='%s',destination ='%s'", MICROFEED_DBUS_INTERFACE_ERROR_TO_DESTINATION, dbus_bus_get_unique_name(subscriber->connection));
+	dbus_bus_add_match(subscriber->connection, buffer, NULL);
 	
 	return subscriber;
 }
@@ -171,6 +190,13 @@ MicrofeedSubscriber* microfeed_subscriber_new(const char* identifier, const char
  * TODO: This function is not yet implemented.
  */
 void microfeed_subscriber_free(MicrofeedSubscriber* subscriber) {
+	char buffer[1024];
+
+	snprintf(buffer, 1024, "type='signal',interface='%s',destination='%s'", MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION, dbus_bus_get_unique_name(subscriber->connection));
+	dbus_bus_remove_match(subscriber->connection, buffer, NULL);
+	snprintf(buffer, 1024, "type='signal',interface='%s',destination ='%s'", MICROFEED_DBUS_INTERFACE_ERROR_TO_DESTINATION, dbus_bus_get_unique_name(subscriber->connection));
+	dbus_bus_remove_match(subscriber->connection, buffer, NULL);
+
 	/* TODO */
 }
 
@@ -258,19 +284,28 @@ int microfeed_subscriber_remove_item(MicrofeedSubscriber* subscriber, const char
 
 }
 
-int microfeed_subscriber_subscribe_feed(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, MicrofeedSubscriberCallbacks* callbacks, MicrofeedSubscriberErrorCallback callback, void* user_data) {
+int microfeed_subscriber_subscribe_feed(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, MicrofeedSubscriberCallbacks* callbacks, void* callbacks_user_data, MicrofeedSubscriberErrorCallback callback, void* user_data) {
 	int retvalue = 0;
 	Publisher* publisher;
 	Feed* feed;
 	DBusMessage* message;
+	char buffer[1024];
 
 	if ((publisher = get_publisher(subscriber, publisher_identifier, uri, NULL, callback, user_data))) {
 		if ((feed = microfeed_store_get(publisher->feeds, uri, Feed))) {
+			feed_add_subscription(feed, callbacks, callbacks_user_data);
 			if (callback) {
-				callback(subscriber, publisher_identifier, uri, NULL, MICROFEED_ERROR_FEED_ALREADY_SUBSCRIBED, "Subscribed a feed that is already subscribed.", user_data);
+				callback(subscriber, publisher_identifier, uri, NULL, NULL, NULL, user_data);
 			}
 		} else {
-			feed = feed_new(publisher, uri, callbacks, user_data);
+			feed = feed_new(publisher, uri);
+			feed_add_subscription(feed, callbacks, callbacks_user_data);
+
+			snprintf(buffer, 1024, "type='signal',path='%s',interface='%s',arg0='%s'", publisher->object_path, MICROFEED_DBUS_INTERFACE_PUBLISHER, uri);
+			printf("%s : %d\n", buffer);
+			dbus_bus_add_match(subscriber->connection, buffer, NULL);
+			snprintf(buffer, 1024, "type='signal',path='%s',interface='%s',arg0='%s'", publisher->object_path, MICROFEED_DBUS_INTERFACE_ERROR, uri);
+			dbus_bus_add_match(subscriber->connection, buffer, NULL);
 		
 			message = dbus_message_new_method_call(feed->publisher->provider->bus_name, feed->publisher->object_path, MICROFEED_DBUS_INTERFACE_PUBLISHER, MICROFEED_METHOD_NAME_SUBSCRIBE_FEED);
 			dbus_message_append_args(message, DBUS_TYPE_STRING, &feed->uri, DBUS_TYPE_INVALID);
@@ -283,23 +318,37 @@ int microfeed_subscriber_subscribe_feed(MicrofeedSubscriber* subscriber, const c
 	return retvalue;
 }
 
-int microfeed_subscriber_unsubscribe_feed(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, MicrofeedSubscriberErrorCallback callback, void* user_data) {
+int microfeed_subscriber_unsubscribe_feed(MicrofeedSubscriber* subscriber, const char* publisher_identifier, const char* uri, MicrofeedSubscriberCallbacks* callbacks, void* callbacks_user_data, MicrofeedSubscriberErrorCallback callback, void* user_data) {
 	int retvalue = 0;
 	Feed* feed;
-	DBusMessage* message;
 	Publisher* publisher;
+	DBusMessage* message;
+	char buffer[1024];
 	
 	if ((feed = get_feed(subscriber, publisher_identifier, uri, NULL, callback, user_data))) {
-		message = dbus_message_new_method_call(feed->publisher->provider->bus_name, feed->publisher->object_path, MICROFEED_DBUS_INTERFACE_PUBLISHER, MICROFEED_METHOD_NAME_UNSUBSCRIBE_FEED);
-		dbus_message_append_args(message, DBUS_TYPE_STRING, &feed->uri, DBUS_TYPE_INVALID);
-		call_publisher_method(feed->publisher, feed->uri, NULL, message, callback, user_data);
-		dbus_message_unref(message);
-		retvalue = 1;
-	
 		publisher = feed->publisher;
-		feed_free(feed);
-		if (microfeed_store_get_size(publisher->feeds) == 0) {
-			publisher_unref(publisher);
+		feed_remove_subscription(feed, callbacks, callbacks_user_data);
+		if (feed->subscriptions) {
+			if (callback) {
+				callback(subscriber, publisher_identifier, uri, NULL, NULL, NULL, user_data);
+			}
+		} else {
+			snprintf(buffer, 1024, "type='signal',path='%s',interface='%s',arg0='%s'", publisher->object_path, MICROFEED_DBUS_INTERFACE_PUBLISHER, uri);
+			dbus_bus_remove_match(subscriber->connection, buffer, NULL);
+			snprintf(buffer, 1024, "type='signal',path='%s',interface='%s',arg0='%s'", publisher->object_path, MICROFEED_DBUS_INTERFACE_ERROR, uri);
+			dbus_bus_remove_match(subscriber->connection, buffer, NULL);
+
+			message = dbus_message_new_method_call(feed->publisher->provider->bus_name, feed->publisher->object_path, MICROFEED_DBUS_INTERFACE_PUBLISHER, MICROFEED_METHOD_NAME_UNSUBSCRIBE_FEED);
+			dbus_message_append_args(message, DBUS_TYPE_STRING, &feed->uri, DBUS_TYPE_INVALID);
+			call_publisher_method(feed->publisher, feed->uri, NULL, message, callback, user_data);
+			dbus_message_unref(message);
+			retvalue = 1;
+
+			publisher = feed->publisher;
+			feed_free(feed);
+			if (microfeed_store_get_size(publisher->feeds) == 0) {
+				publisher_unref(publisher);
+			}
 		}
 	}
 	
@@ -461,6 +510,7 @@ int microfeed_subscriber_handle_configured_subscriptions(MicrofeedSubscriber* su
 	Publisher* publisher;
 	MicrofeedStoreIterator* feeds_iterator;
 	Feed* feed;
+	Subscription* subscription;
 	
 	microfeed_configuration_invalidate(subscriber->configuration);
 	if ((subscriptions = microfeed_configuration_get_subscriptions(subscriber->configuration, subscriber->identifier))) {
@@ -486,7 +536,9 @@ int microfeed_subscriber_handle_configured_subscriptions(MicrofeedSubscriber* su
 				for (feeds_iterator = microfeed_store_iterate(publisher->feeds, NULL);
 				     (feed = microfeed_store_iterator_get(feeds_iterator, Feed));
 				     microfeed_store_iterator_next(feeds_iterator)) {
-					configured_unsubscribe(subscriber, publisher->identifier, feed->uri, user_data);
+					for (subscription = feed->subscriptions; subscription; subscription = subscription->next) {
+						configured_unsubscribe(subscriber, publisher->identifier, feed->uri, subscription->callbacks, subscription->user_data, user_data);
+					}
 				}
 			}
 		}
@@ -545,24 +597,9 @@ static void provider_set_unique_connection_name(Provider* provider, const char*
 
 		snprintf(buffer, 1024, "type='signal',sender='org.freedesktop.DBus',interface='org.freedesktop.DBus',member='NameOwnerChanged',arg0='%s'", provider->unique_connection_name);
 		dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
-		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_PUBLISHER);
-		dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
-		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s',destination='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION, dbus_bus_get_unique_name(provider->subscriber->connection));
-		dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
-		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_ERROR);
-		dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
-		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s',destination ='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_ERROR_TO_DESTINATION, dbus_bus_get_unique_name(provider->subscriber->connection));
-		dbus_bus_add_match(provider->subscriber->connection, buffer, NULL);
 	} else if (provider->unique_connection_name) {
 		snprintf(buffer, 1024, "type='signal',sender='org.freedesktop.DBus',interface='org.freedesktop.DBus',member='NameOwnerChanged',arg0='%s'", provider->unique_connection_name);
 		dbus_bus_remove_match(provider->subscriber->connection, buffer, NULL);
-		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_PUBLISHER);
-		dbus_bus_remove_match(provider->subscriber->connection, buffer, NULL);
-		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s',destination='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_PUBLISHER_TO_DESTINATION, dbus_bus_get_unique_name(provider->subscriber->connection));
-		dbus_bus_remove_match(provider->subscriber->connection, buffer, NULL);
-		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_ERROR);
-		dbus_bus_remove_match(provider->subscriber->connection, buffer, NULL);
-		snprintf(buffer, 1024, "type='signal',sender='%s',interface='%s',destination='%s'", provider->unique_connection_name, MICROFEED_DBUS_INTERFACE_ERROR_TO_DESTINATION, dbus_bus_get_unique_name(provider->subscriber->connection));
 
 		microfeed_store_remove(provider->subscriber->providers_by_unique_connection_name, provider);
 		free(provider->unique_connection_name);
@@ -610,13 +647,13 @@ static void publisher_free(Publisher* publisher) {
 	microfeed_memory_free(publisher);
 }
 
-Publisher* publisher_ref(Publisher* publisher) {
+static Publisher* publisher_ref(Publisher* publisher) {
 	publisher->reference_count++;
 	
 	return publisher;
 }
 
-void publisher_unref(Publisher* publisher) {
+static void publisher_unref(Publisher* publisher) {
 	publisher->reference_count--;
 	if (publisher->reference_count == 0) {
 		publisher_free(publisher);
@@ -633,15 +670,13 @@ static const char* publisher_get_object_path(Publisher* publisher) {
 	return publisher->object_path;
 }
 
-static Feed* feed_new(Publisher* publisher, const char* uri, MicrofeedSubscriberCallbacks* callbacks, void* user_data) {
+static Feed* feed_new(Publisher* publisher, const char* uri) {
 	Feed* feed;
 	
 	feed = microfeed_memory_allocate(Feed);
 	feed->reference_count = 1;
 	feed->publisher = publisher;
 	feed->uri = strdup(uri);
-	feed->callbacks = *callbacks;
-	feed->user_data = user_data;
 
 	microfeed_store_insert(publisher->feeds, feed);
 	
@@ -656,7 +691,6 @@ static void feed_free(Feed* feed) {
 	feed->publisher = NULL;
 	feed->uri = NULL;
 	feed->name = NULL;
-	feed->user_data = NULL;
 	microfeed_memory_free(feed);
 }
 /*
@@ -679,35 +713,95 @@ static const char* feed_get_uri(Feed* feed) {
 	return feed->uri;
 }
 
+static void feed_add_subscription(Feed* feed, MicrofeedSubscriberCallbacks* callbacks, void* user_data) {
+	Subscription* s;
+	
+	for (s = feed->subscriptions; s; s = s->next) {
+		if (s->callbacks == callbacks && s->user_data == user_data) {
+			break;
+		}
+	}
+	if (!s) {
+		s = microfeed_memory_allocate(Subscription);
+		s->callbacks = callbacks;
+		s->user_data = user_data;
+		
+		s->next = feed->subscriptions;
+		feed->subscriptions = s;
+	}
+}
+
+static void feed_remove_subscription(Feed* feed, MicrofeedSubscriberCallbacks* callbacks, void* user_data) {
+	Subscription* s;
+	
+	for (s = feed->subscriptions; s; s = s->next) {
+		if (s->callbacks == callbacks && s->user_data == user_data) {
+			break;
+		}
+	}
+	if (s) {
+		if (s->next) {
+			s->next->previous = s->previous;
+		}
+		if (s->previous) {
+			s->previous->next = s->next;
+		} else {
+			feed->subscriptions = s->next;
+		}
+
+		microfeed_memory_free(s);
+	}
+}
+
 static void signal_feed_update_started(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message) {
 	Feed* feed;
+	Subscription* s;
 	
-	if (parse_feed_signal(publisher, message, &feed) && feed->callbacks.feed_update_started) {
-		feed->callbacks.feed_update_started(subscriber, publisher->identifier, feed->uri, feed->user_data);
+	if (parse_feed_signal(publisher, message, &feed)) {
+		for (s = feed->subscriptions; s; s = s->next) {
+			if (s->callbacks->feed_update_started) {
+				s->callbacks->feed_update_started(subscriber, publisher->identifier, feed->uri, s->user_data);
+			}
+		}
 	}
 }
 
 static void signal_feed_update_ended(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message) {
 	Feed* feed;
+	Subscription* s;
 	
-	if (parse_feed_signal(publisher, message, &feed) && feed->callbacks.feed_update_ended) {
-		feed->callbacks.feed_update_ended(subscriber, publisher->identifier, feed->uri, feed->user_data);
+	if (parse_feed_signal(publisher, message, &feed)) {
+		for (s = feed->subscriptions; s; s = s->next) {
+			if (s->callbacks->feed_update_ended) {
+				s->callbacks->feed_update_ended(subscriber, publisher->identifier, feed->uri, s->user_data);
+			}
+		}
 	}
 }
 
 static void signal_feed_republishing_started(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message) {
 	Feed* feed;
+	Subscription* s;
 	
-	if (parse_feed_signal(publisher, message, &feed) && feed->callbacks.feed_republishing_started) {
-		feed->callbacks.feed_republishing_started(subscriber, publisher->identifier, feed->uri, feed->user_data);
+	if (parse_feed_signal(publisher, message, &feed)) {
+		for (s = feed->subscriptions; s; s = s->next) {
+			if (s->callbacks->feed_republishing_started) {
+				s->callbacks->feed_republishing_started(subscriber, publisher->identifier, feed->uri, s->user_data);
+			}
+		}
 	}
 }
 
 static void signal_feed_republishing_ended(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message) {
 	Feed* feed;
+	Subscription* s;
 	
-	if (parse_feed_signal(publisher, message, &feed) && feed->callbacks.feed_republishing_ended) {
-		feed->callbacks.feed_republishing_ended(subscriber, publisher->identifier, feed->uri, feed->user_data);
+	if (parse_feed_signal(publisher, message, &feed)) {
+		for (s = feed->subscriptions; s; s = s->next) {
+			if (s->callbacks->feed_republishing_ended) {
+				s->callbacks->feed_republishing_ended(subscriber, publisher->identifier, feed->uri, s->user_data);
+			}
+		}
 	}
 }
 
@@ -719,17 +813,23 @@ static void signal_item_data(MicrofeedSubscriber* subscriber, Publisher* publish
 	DBusMessageIter subiter;
 	char* data;
 	int length;
+	Subscription* s;
 		
 	if (dbus_message_iter_init(message, &iter) && dbus_message_iter_get_arg_type(&iter) == DBUS_TYPE_STRING) {
 		dbus_message_iter_get_basic(&iter, &uri);
-		if ((feed = microfeed_store_get(publisher->feeds, uri, Feed)) && feed->callbacks.item_data_received &&
+		if ((feed = microfeed_store_get(publisher->feeds, uri, Feed)) &&
 		    dbus_message_iter_next(&iter) && dbus_message_iter_get_arg_type(&iter) == DBUS_TYPE_STRING) {
 			dbus_message_iter_get_basic(&iter, &uid);
 			if (dbus_message_iter_next(&iter) && dbus_message_iter_get_arg_type(&iter) == DBUS_TYPE_ARRAY &&
 			    dbus_message_iter_get_element_type(&iter) == DBUS_TYPE_BYTE) {
 				dbus_message_iter_recurse(&iter, &subiter);
 				dbus_message_iter_get_fixed_array(&subiter, &data, &length);
-				feed->callbacks.item_data_received(subscriber, publisher->identifier, uri, uid, data, length, feed->user_data);
+	
+				for (s = feed->subscriptions; s; s = s->next) {
+					if (s->callbacks->item_data_received) {
+						s->callbacks->item_data_received(subscriber, publisher->identifier, feed->uri, uid, data, length, s->user_data);
+					}
+				}
 			}
 		}
 	}
@@ -738,9 +838,14 @@ static void signal_item_data(MicrofeedSubscriber* subscriber, Publisher* publish
 static void signal_item_added(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message) {
 	Feed* feed;
 	MicrofeedItem* item;
+	Subscription* s;
 	
-	if (parse_item_signal(publisher, message, &feed, &item) && feed->callbacks.item_added) {
-		feed->callbacks.item_added(subscriber, publisher->identifier, feed->uri, item, feed->user_data);
+	if (parse_item_signal(publisher, message, &feed, &item)) {
+		for (s = feed->subscriptions; s; s = s->next) {
+			if (s->callbacks->item_added) {
+				s->callbacks->item_added(subscriber, publisher->identifier, feed->uri, item, s->user_data);
+			}
+		}
 		microfeed_item_free(item);
 	}
 }
@@ -748,9 +853,14 @@ static void signal_item_added(MicrofeedSubscriber* subscriber, Publisher* publis
 static void signal_item_changed(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message) {
 	Feed* feed;
 	MicrofeedItem* item;
+	Subscription* s;
 	
-	if (parse_item_signal(publisher, message, &feed, &item) && feed->callbacks.item_changed) {
-		feed->callbacks.item_changed(subscriber, publisher->identifier, feed->uri, item, feed->user_data);
+	if (parse_item_signal(publisher, message, &feed, &item)) {
+		for (s = feed->subscriptions; s; s = s->next) {
+			if (s->callbacks->item_changed) {
+				s->callbacks->item_changed(subscriber, publisher->identifier, feed->uri, item, s->user_data);
+			}
+		}
 		microfeed_item_free(item);
 	}
 }
@@ -758,9 +868,14 @@ static void signal_item_changed(MicrofeedSubscriber* subscriber, Publisher* publ
 static void signal_item_republished(MicrofeedSubscriber* subscriber, Publisher* publisher, DBusMessage* message) {
 	Feed* feed;
 	MicrofeedItem* item;
+	Subscription* s;
 	
-	if (parse_item_signal(publisher, message, &feed, &item) && feed->callbacks.item_republished) {
-		feed->callbacks.item_republished(subscriber, publisher->identifier, feed->uri, item, feed->user_data);
+	if (parse_item_signal(publisher, message, &feed, &item)) {
+		for (s = feed->subscriptions; s; s = s->next) {
+			if (s->callbacks->item_republished) {
+				s->callbacks->item_republished(subscriber, publisher->identifier, feed->uri, item, s->user_data);
+			}
+		}
 		microfeed_item_free(item);
 	}
 }
@@ -770,11 +885,16 @@ static void signal_item_removed(MicrofeedSubscriber* subscriber, Publisher* publ
 	char* uri;
 	char* uid;
 	Feed* feed;
+	Subscription* s;
 	
 	dbus_error_init(&error);
 	if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &uri, DBUS_TYPE_STRING, &uid, DBUS_TYPE_INVALID) &&
-	    (feed = microfeed_store_get(publisher->feeds, uri, Feed)) && feed->callbacks.item_removed) {
-		feed->callbacks.item_removed(subscriber, publisher->identifier, uri, uid, feed->user_data);
+	    (feed = microfeed_store_get(publisher->feeds, uri, Feed))) {
+		for (s = feed->subscriptions; s; s = s->next) {
+			if (s->callbacks->item_republished) {
+				s->callbacks->item_removed(subscriber, publisher->identifier, feed->uri, uid, s->user_data);
+			}
+		}
 	}
 }
 
@@ -784,11 +904,16 @@ static void signal_item_status_changed(MicrofeedSubscriber* subscriber, Publishe
 	char* uid;
 	char status;
 	Feed* feed;
+	Subscription* s;
 	
 	dbus_error_init(&error);
 	if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &uri, DBUS_TYPE_STRING, &uid, DBUS_TYPE_BYTE, &status, DBUS_TYPE_INVALID) &&
-	    (feed = microfeed_store_get(publisher->feeds, uri, Feed)) && feed->callbacks.item_status_changed) {
-		feed->callbacks.item_status_changed(subscriber, publisher->identifier, uri, uid, (MicrofeedItemStatus)status, feed->user_data);
+	    (feed = microfeed_store_get(publisher->feeds, uri, Feed))) {
+		for (s = feed->subscriptions; s; s = s->next) {
+			if (s->callbacks->item_status_changed) {
+				s->callbacks->item_status_changed(subscriber, publisher->identifier, feed->uri, uid, (MicrofeedItemStatus)status, s->user_data);
+			}
+		}
 	}
 }
 
@@ -929,7 +1054,7 @@ static void handle_publisher_method_return(DBusPendingCall* pending, void* user_
 			}
 			data->retry_counter++;
 			
-			dbus_connection_send_with_reply(data->publisher->provider->subscriber->connection, data->message, &pending_call, -1);
+			dbus_connection_send_with_reply(data->publisher->provider->subscriber->connection, data->message, &pending_call, MICROFEED_SUBSCRIBER_DBUS_METHOD_CALL_TIMEOUT);
 			if (pending_call) {
 				dbus_pending_call_set_notify(pending_call, handle_publisher_method_return, data, NULL);
 			}
@@ -968,7 +1093,7 @@ static void call_publisher_method(Publisher* publisher, const char* uri, const c
 	DBusPendingCall* pending_call;
 	MethodReturnData* data;
 	
-	dbus_connection_send_with_reply(publisher->provider->subscriber->connection, message, &pending_call, -1);
+	dbus_connection_send_with_reply(publisher->provider->subscriber->connection, message, &pending_call, MICROFEED_SUBSCRIBER_DBUS_METHOD_CALL_TIMEOUT);
 	if (pending_call) {
 		data = microfeed_memory_allocate(MethodReturnData);
 		data->message = dbus_message_copy(message);
@@ -1010,6 +1135,7 @@ static DBusHandlerResult object_message(DBusConnection* connection, DBusMessage*
 	char* error_message;
 	char* uri;
 	char* uid;
+	Subscription* s;
 
 	subscriber = (MicrofeedSubscriber*)user_data;
 	if (dbus_message_is_signal(message, "org.freedesktop.DBus", "NameOwnerChanged") &&
@@ -1021,8 +1147,10 @@ static DBusHandlerResult object_message(DBusConnection* connection, DBusMessage*
 			for (feed_iterator = microfeed_store_iterate(publisher->feeds, NULL);
 			     (feed = microfeed_store_iterator_get(feed_iterator, Feed));
 			     microfeed_store_iterator_next(feed_iterator)) {
-				if (feed->callbacks.error_occured) {
-					feed->callbacks.error_occured(subscriber, publisher->identifier, feed->uri, NULL, MICROFEED_ERROR_PROVIDER_CLOSED_CONNECTION, "Provider closed connection", feed->user_data);	
+				for (s = feed->subscriptions; s; s = s->next) {
+					if (s->callbacks->error_occured) {
+						s->callbacks->error_occured(subscriber, publisher->identifier, feed->uri, NULL, MICROFEED_ERROR_PROVIDER_CLOSED_CONNECTION, "Provider closed connection", s->user_data);	
+					}
 				}
 				/* TODO: Think! Think! Think! feed_free(feed); */
 			}
@@ -1044,8 +1172,12 @@ static DBusHandlerResult object_message(DBusConnection* connection, DBusMessage*
 		} else if (dbus_message_get_type(message) == DBUS_MESSAGE_TYPE_SIGNAL && (dbus_message_has_interface(message, MICROFEED_DBUS_INTERFACE_ERROR) || dbus_message_has_interface(message, MICROFEED_DBUS_INTERFACE_ERROR_TO_DESTINATION))) {
 			dbus_error_init(&error);
 			if ((error_name = dbus_message_get_member(message)) && dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &uri, DBUS_TYPE_STRING, &uid, DBUS_TYPE_STRING, &error_message, DBUS_TYPE_INVALID)) {
-				if ((feed = microfeed_store_get(publisher->feeds, uri, Feed)) && feed->callbacks.error_occured) { 
-					feed->callbacks.error_occured(subscriber, publisher->identifier, uri, uid, error_name, error_message, feed->user_data);
+				if ((feed = microfeed_store_get(publisher->feeds, uri, Feed))) { 
+					for (s = feed->subscriptions; s; s = s->next) {
+						if (s->callbacks->error_occured) {
+							s->callbacks->error_occured(subscriber, publisher->identifier, uri, uid, error_name, error_message, s->user_data);
+						}
+					}
 				}
 			}
 		} else {
@@ -1062,4 +1194,3 @@ static DBusHandlerResult object_message(DBusConnection* connection, DBusMessage*
 	
 	return result;
 }
-
